package Modules::AddrBook;

#----------------------------------------------------------------------------
#
#   Copyright (C) 2010 - Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
#  The address book manages your database of people and places.
#
#----------------------------------------------------------------------------

use strict;

# useful kernel libs; you can/should remove any of these that are not needed

use ExSite::Config;          # required
use ExSite::Input;           # optional input manager
use ExSite::Misc;            # optional utils
use ExSite::Util;            # optional utils
use ExSite::ML;              # optional markup-language generation
use ExSite::FormBuilder;     # optional form generation
use ExSite::ReportBuilder;   # optional report generation
use ExSite::Form;            # optional database manager

use Modules::ID::Contact;
use Modules::ID::ContactList;
use Modules::ID::Location;

# recommended base class for plug-in modules

use Modules::BaseDCD;

# declare package globals

use vars qw(@ISA $ml);       # Perl inheritance global & markup language object

# define our class inheritance

@ISA = qw(Modules::BaseDCD); # inherit from this base class



# ioctl method (used by ExSite to query the module for its functionality)

sub ioctl {
    my $this = shift;
    $_ = shift;           # $_ is the ioctl request

    if (/isRestricted/) {
	return 0;
    }
    elsif (/isService/) {
	return 0;
    }
    elsif (/ModuleName/) {
	return "Address Book";
    }
    elsif (/ModuleInfo/) {
	return "The address book manages your database of people and places.";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/Publish/) {
	return \&publish;
    }
    elsif (/Unpublish/) {
	return \&unpublish;
    }
    elsif (/Dependencies/) {
	return ["EvtCal","EvtReg","Finance"];
    }
}

#----------------------------------------------------------------------------
# Everything after this point consists of private methods.

# ctrl_panel() generates the contents of the administrator control panel

sub ctrl_panel {
    my $this = shift;

    # declare a markup generator
    $ml = new ExSite::ML;

    # build a block of HTML to display the control panel

    my $out;  # our output buffer
    $this->{section_id} = $this->get_section_id();
    if (! $this->{section_id}) {
	$out .= $this->set_section_id();
	return $out;
    }

    $this->setup_querylib();

    my %tab = (
	       contact => 0,
	       saved => 1,
	       location => 2,
	       update => 3,
	       );

    my $tab = $this->{input}{tab} || "contact";

    my %basequery = (section_id=>$this->{input}{section_id});
    my $baseuri = "$ENV{SCRIPT_NAME}$ENV{PATH_INFO}?".&EncodeHash(%basequery);
    my $link = new ExSite::URI(uri=>$baseuri);
    $out .= &ExSite::HTML::TabBox(
				  tabs=>[
					 {label=>"Contact Lists",url=>$link->relink(query=>{tab=>"contact"})},
					 {label=>"Saved Lists",url=>$link->relink(query=>{tab=>"saved"})},
					 {label=>"Locations",url=>$link->relink(query=>{tab=>"location"})},
					 {label=>"Update",url=>$link->relink(query=>{tab=>"update"})},
					 ],
				  thistab=>$tab{$tab},
				  );
    if ($tab eq "contact") {
	$out .= $this->contacts();
    }
    elsif ($tab eq "saved") {
	$out .= $this->saved_lists();
    }
    elsif ($tab eq "location") {
	$out .= $this->locations();
    }
    elsif ($tab eq "update") {
	$out .= $this->update();
    }

    return $out;
}

sub contacts {
    my ($this) = @_;
    my $out;
    my $in = $this->{input};
    my $section_id = $this->{section_id};
    if ($in->{ab} eq "new") {
	return $this->new_contact();
    }

    my %tab = (
	       user => 0,
	       customer => 1,
	       registrant => 2,
	       location => 3,
	       other => 4,
	       );

    my $tab = $this->{input}{ctype} || "user";
    
    my %basequery = (tab=>$in->{tab},section_id=>$in->{section_id});
    my $baseuri = "$ENV{SCRIPT_NAME}$ENV{PATH_INFO}?".&EncodeHash(%basequery);
    my $link = new ExSite::URI(uri=>$baseuri);
    $out .= &ExSite::HTML::TabBox(
				  tabs=>[
					 {label=>"Users",url=>$link->relink(query=>{ctype=>"user"})},
					 {label=>"Customers",url=>$link->relink(query=>{ctype=>"customer"})},
					 {label=>"Registrants",url=>$link->relink(query=>{ctype=>"registrant"})},
					 {label=>"Locations",url=>$link->relink(query=>{ctype=>"location"})},
					 {label=>"Other",url=>$link->relink(query=>{ctype=>"other"})},
					 ],
				  thistab=>$tab{$tab},
				  );

    $out .= $this->contact_search_form($tab);

    if ($in->{filter}) {
	return $this->save_contact_list() if ($in->{save});
	
	my $clist = $this->get_contact_list();
	my $r = new ExSite::ReportBuilder(title=>ucfirst($in->{ctype})." contacts");
	$r->nodata("No matching contact records.");
	$r->headers(["Name","Contact","Address","City","Prov/State","Country","Postal Code","Phone","Phone/Cell","Fax","E-mail","URL","Privacy","Notes","Type"]);
	while (my $c = $clist->next()) {
	    my $editurl = $this->link(tab=>"update",contact=>$c->id);
	    my @data = (
			$ml->a($this->contact_name($c),{href=>$editurl}),
			$c->showdata("type"),
			$c->showdata("address"),
			$c->showdata("city"),
			$c->showdata("provstate"),
			$c->showdata("country"),
			$c->showdata("pcode"),
			$c->showdata("phone1"),
			$c->showdata("phone2"),
			$c->showdata("fax"),
			$c->showdata("email"),
			$c->showdata("website"),
			$c->showdata("privacy"),
			$c->showdata("notes"),
			$c->showdata("type"),
			);
	    $r->push(@data);
	}
	if ($in->{export} eq "xls") { 
	    $ml->cancel;
	    print "Content-type: application/vnd.ms-excel\ncontent-disposition: attachment; filename=query.xls\n\n";
	    $r->export("excel");
	    return undef;
	}
	elsif ($in->{export} eq "csv") {
	    $ml->cancel;
	    print "Content-type: text/comma-separated-values\n\n";
	    print $r->export("csv");
	    return undef;
	}
	else {
	    # HTML
	    $r->paginate(1000);
	    $r->tools( $ml->a("+ add contact",{href=>$this->link(ab=>"new")}) );
	    # would be nice to force a filename on CSV exports
	    $r->foot(
		     $ml->a("CSV",{href=>$this->link(export=>"csv")}).
		     "&nbsp;&bull;&nbsp;".
		     $ml->a("Excel",{href=>$this->link(export=>"xls")}).
		     "&nbsp;&bull;&nbsp;".
		     $ml->a("Save",{href=>$this->link(save=>1)})
		     );
	    $out .= $r->make();
	}
    }
    return $out;
}

sub contact_name {
    my ($this, $c) = @_;

    my $name = $c->getdata("name");
    return $name if $name;

    my $first_name = $c->getdata("first_name");
    my $last_name = $c->getdata("last_name");
    if ($first_name || $last_name) {
	return ($last_name || "?") . ", " . $first_name;
    }

    return $c->contact_name;
}

sub save_contact_list {
    my $this = shift;
    if ($this->{input}{save} eq "1") {
	my $out = $ml->h1("Save Contact List");
	$out .= $ml->p("NOTE: This saves the parameters used to generate the contact list, 
not the actual contents of the list. That means the contact list may have 
different contents when viewed at different times.");
	$out .= $ml->p("To save the current contents of the contact list, you
should export the list to Excel or CSV.");
	my $f = new ExSite::FormBuilder();
	$f->input(name=>"save",prompt=>"Save contact list as",type=>"text",required=>1);
	$out .= $f->make();
	return $out;
    }
    else {
	my %param = %{$this->{input}};
	delete $param{filter};
	my $section_id = $param{section_id};
	delete $param{section_id};
	my $dupe = $param{dupe};
	my $unsubscribe = $param{unsubscribe};
	delete $param{dupe};
	delete $param{unsubscribe};
	my $type = $param{ctype};
	delete $param{ctype};
	my $name = $param{save};
	delete $param{save};
	my %contact_list_type = ( user=>"member", customer=>"customer", registrant=>"registrant", location=>"location", other=>"other" );
	my %contact_list = (
			    section_id=>$section_id,
			    name=>$name,
			    type=>$contact_list_type{$type},
			    dupe=>$dupe,
			    unsubscribe=>$unsubscribe,
			    parameters=>&EncodeHash(%param),
			    );
	if ($share{DB}->insert("contact_list",\%contact_list)) {
	    return $ml->p("Contact list saved.");
	}
	else {
	    return $this->error("Failed to save contact list.").
		$share{DB}->show_diagnostics("error","html");
	}
    }
}

sub saved_lists {
    my $this = shift;
    my $out .= $ml->h1("Saved Contact Lists");
    my $r = new ExSite::ReportBuilder(title=>"Saved Contact Lists");
    $r->headers("Name","Type");
    $r->nodata("No saved contact lists.");
    my $section_id = $this->get_section_id;
    my @clist = $share{DB}->fetch_child("contact_list","section",$section_id,"name");
    my %ctype = (member=>"user",membership=>"user",'membership-extended'=>"user",customer=>"customer",registrant=>"registrant",location=>"location",other=>"other");
    
    foreach my $clist (@clist) {
	my %arg = &DecodeString($clist->{parameters});
	$arg{filter} = 1;
	$arg{ctype} = $ctype{$clist->{type}};
	$r->push(
		 $ml->a($clist->{name},{href=>$this->link(tab=>undef,%arg)}),
		 $clist->{type}
		 )
    }
    $out .= $r->make();
}

sub contact_search_form {
    my ($this,$type) = @_;
    my $in = $this->{input};
    my $out .= $ml->h2("Select $type contact records");
    if ($type eq "user") {
	$out .= $ml->p("Users are persons/organizations with login accounts on your website.");
    }
    elsif ($type eq "customer") {
	$out .= $ml->p("Customers are members of the public who have purchased from your website.");
    }
    elsif ($type eq "registrant") {
	$out .= $ml->p("Registrants are users who have purchased event registrations from your website.");
    }
    my $f = new ExSite::FormBuilder(action=>$this->link(),method=>"get");

    if ($type eq "user") {
	my $membership = &ExSite::Module::get_module("Membership");
	if ($membership) {
	    $membership->setup_queries();
	    my @utype = sort $membership->known_membership_types($this->get_section_id());
	    if (@utype > 0) {
		$f->input(name=>"usertype",prompt=>"User Type",type=>"select",options=>\@utype,nullvalue=>"ANY",value=>$in->{usertype});
	    }
	    $f->input(name=>"status",prompt=>"Membership Status",type=>"select",options=>["active","pending","expired","archived"],nullvalue=>"ANY",value=>$in->{status});
	}
    }
#    elsif ($type eq "customer") {
#	my @code;
#	foreach my $ac ($share{DB}->fetch_all("acctcode","code")) {
#	    push @code, [$ac->{acctcode_id},$ac->{code}];
#	}
#	$f->input(name=>"acctcode",prompt=>"Purchase Type",type=>"select",options=>\@code,nullvalue=>"ANY",value=>$in->{acctcode});
#	$f->input(name=>"startdate",prompt=>"From",type=>"text",placeholder=>"YYYY-MM-DD",value=>$in->{startdate},size=>12);
#	$f->input(name=>"enddate",prompt=>"To",type=>"text",placeholder=>"YYYY-MM-DD",value=>$in->{enddate},size=>12);
#    }
    elsif ($type eq "registrant") {
	my (@evt,%evt,$last_evt,$last_evt_id);
	foreach my $evt ($share{DB}->get_query("events with registrant contacts",$this->{input}{section_id})) {
	    if ($evt->{evt_id} != $last_evt && $evt{$last_evt} > 1) {
		push @evt, ["$last_evt-0",$evt->{event}."/ALL FEES (".$evt->{date}.")"];
	    }
	    push @evt, [$evt->{evt_id}."-".$evt->{evt_fee_id},$evt->{event}."/".$evt->{fee}." (".$evt->{date}.")"];	
	    $last_evt = $evt->{event};
	    $last_evt_id = $evt->{evt_id};
	    $evt{$evt->{evt_id}}++;
	}
	$f->input(name=>"evtfee",prompt=>"Event/Fee",type=>"select",options=>\@evt,nullvalue=>"ANY",value=>$in->{evt});
	$f->input(name=>"status",prompt=>"Registration Status",type=>"select",options=>$share{DB}{map}->regexp("list:evt_reg_status"),nullvalue=>"ANY",value=>$in->{status});
    }
    elsif ($type eq "other") {
	my @group = split /\|/, $share{DB}{map}->regexp("list:contact_group");
	if (@group > 0) {
	    $f->input(name=>"group",prompt=>"Group",type=>"select",options=>\@group,nullvalue=>"ANY",value=>$in->{group});
	}
    }

    $f->input(name=>"type",prompt=>"Contact Type",type=>"select",options=>$share{DB}{map}->regexp("list:contact_type"),value=>$in->{type},nullvalue=>"ANY");
    $f->input(name=>"provstate",prompt=>"Province/State",type=>"select",options=>$share{DB}{map}->regexp("list:provstate"),value=>$in->{provstate},nullvalue=>"ANY");
    $f->input(name=>"country",prompt=>"Country",type=>"text",value=>$in->{country},nullvalue=>"ANY",placeholder=>"ANY");
    $f->input(name=>"dupe",prompt=>"Contact records per person",type=>"select",options=>["all","best one"],value=>$in->{dupe});
    $f->input(name=>"unsubscribe",prompt=>"Unsubscribed contact records",type=>"select",options=>["exclude","include"],value=>$in->{unsubscribe});
    $f->input(name=>"filter",type=>"hidden",value=>1);
    $f->input(name=>"ctype",type=>"hidden",value=>$type);
    $f->input(name=>"section_id",type=>"hidden",value=>$this->get_section_id());
    $f->set("buttons",
	    $ml->input(undef,{type=>"submit",value=>"Search"})." ".
	    $ml->input(undef,{type=>"reset"})
	    );
    $f->template("<table>","<tr><td>[[prompt]]</td><td>[[input]]</td></tr>","</table>");
    $out .= $f->make();
    return $out;
}

sub get_contact_list {
    my ($this) = @_;
    my $in = $this->{input};
    my $out;

    my $clist = new Modules::ID::ContactList();
    my %match;
    my $method;
    if ($in->{provstate} && $in->{provstate} !~ /==/) {
	$match{provstate} = $in->{provstate};
    }
    if ($in->{country}) {
	$match{country} = $in->{country};
    }
    if ($in->{type}) {
	$match{"c.type"} = $in->{type};
    }
    if ($in->{ctype} eq "user") {
	if ($in->{usertype} || $in->{status}) {
	    $method = "load_membership";
	    if ($in->{usertype}) {
		$match{'member_type'} = $in->{usertype};
	    }
	    if ($in->{status}) {
		$match{status} = $in->{status};
	    }
	}
	else {
	    $method = "load_members";
	}
    }
    elsif ($in->{ctype} eq "customer") {
	$method = "load_customers";
    }
    elsif ($in->{ctype} eq "registrant") {
	my ($evt,$fee) = split /-/,$in->{evtfee};
	$match{evt_id} = $evt;
	if ($fee) { $match{evt_fee_id} = $fee; }
	if ($in->{status}) {
	    $match{'r.status'} = $in->{status};
	}
	$method = "load_registrants";
    }
    elsif ($in->{ctype} eq "location") {
	$method = "load_locations";
    }
    elsif ($in->{ctype} eq "other") {
	if ($in->{group}) {
	    $match{category} = $in->{group};
	}
	$method = "load_other";
    }
    $clist->$method(\%match);
    if ($in->{unsubscribe} eq "exclude") {
	$clist->remove_unsubscribed();
    }
    if ($in->{dupe} =~ /best/) {
	if ($in->{ctype} eq "user") {
	    $clist->best_of_account();
	}
	elsif ($in->{ctype} eq "customer") {
	    $clist->best_of_account();
	}
	elsif ($in->{ctype} eq "location") {
	    $clist->best_of_location();
	}
	elsif ($in->{ctype} eq "other") {
	    $clist->remove_dupe_emails();
	    $clist->remove_dupe_addresses();
	}
    }
    return $clist;
}

sub update {
    my $this = shift;
    my $input = new ExSite::Input();
    my $post = $input->post();
    my $out;
    if (keys %$post > 0) {
	$out .= $this->doform($post,$this->link(ab=>undef,__plaintext=>1));
    }
    if ($this->{input}{ab} eq "new") {
	$out .= $this->new_contact();
    }
    elsif ($this->{input}{contact}) {
	my $c = new Modules::ID::Contact(id=>$this->{input}{contact});
	$out .= $c->show(title=>1,notes=>1);
	if ($this->{input}{ab} eq "edit") {
	    $out .= $this->edit_contact();
	}
	else {
	    $out .= $ml->p($ml->a("Edit",{class=>"button",href=>$this->link(ab=>"edit")}));
	}
    }
    else {
	$out .= $ml->p($ml->a("Create a new contact",{href=>$this->link(ab=>"new",contact=>undef)})); 
    }
    return $out;
}

sub new_contact {
    my $this = shift;
    my $db = $share{DB};
    $db->set_action("insert");
    $db->form(action=>$this->link());
    my $type = $this->{input}{account_id} ? "billing" : "main";
    $db->input_record({
	table=>"contact",
	data=>{
	    type=>$type,
	    section_id=>$this->{section_id},
	    account_id=>$this->{input}{account_id},
	    location_id=>$this->{input}{location_id},
	    country=>$db->{map}->get_column_attr("contact","country","default"),
	},
	show=>["info","type","address","city","provstate","country","pcode","phone1","fax","email","web","category","notes"],
    });
    my $out = $ml->h1("Add Contact");
    if ($this->{input}{account_id}) {
	$out .= $ml->p("Adding contact information to account $this->{input}{account_id}.");
    }
    elsif ($this->{input}{location_id}) {
	$out .= $ml->p("Adding contact information to location $this->{input}{account_id}.");
    }
    else {
	$out .= $ml->p($ml->strong("Note:").
		       " New contacts are filed under 'other'.  To add new contact information for members, use your membership tools.  To add new contacts for locations, go to the Places tab.");
    }
    $out .= $db->form()->make();
    return $out;
}

sub edit_contact {
    my $this = shift;
    my $db = $share{DB};
    my $out = $ml->h1("Update Contact");

    ### FIXME: show what this contact belongs to
    my $contact = new Modules::ID::Contact(id=>$this->{input}{contact});
    $out .= $ml->h3($contact->contact_name);

    $db->set_action("update");
    $db->form(action=>$this->link(contact=>undef));
    $out .= $db->make(
		      table=>"contact",
		      record=>$this->{input}{contact},
		      show=>["info","type","address","city","provstate","country","pcode","phone1","fax","email","web","category","notes"],
		      action=>$this->link(),
    );
    return $out;
}

sub locations {
    my ($this) = @_;
    my $section_id = $this->{section_id};

    my $cmd = $this->{input}{ab};
    if ($cmd eq "new") {
	return $this->new_location();
    }
    elsif ($cmd eq "view") {
	return $this->show_location($this->{input}{lid});
    }
    elsif ($cmd eq "addcon") {
	return $this->add_location_contact($this->{input}{lid});
    }
    elsif ($cmd eq "edit") {
	return $this->edit_location($this->{input}{lid});
    }

    my @location = $share{DB}->get_query("my locations",$section_id);

    my $r = new ExSite::ReportBuilder(title=>"Locations");
    $r->nodata("You have no locations defined in your database.");
    $r->headers(["Location","Address","City","Prov/State","Phone","E-mail","Web"]);
    foreach my $loc (@location) {
	my $l = new Modules::ID::Location(data=>$loc);
	my $c = new Modules::ID::Contact(data=>$loc);
	my @data = (
		    $ml->a($l->name,{href=>$this->link(lid=>$loc->{location_id},ab=>"view")}),
		    $c->showdata("address"),
		    $c->showdata("city"),
		    $c->showdata("provstate"),
		    $c->showdata("phone1"),
		    $c->showdata("email"),
		    $c->showdata("web"),
		    );
	$r->push(@data);
    }
    $r->tools($ml->a("+ new location",{href=>$this->link(ab=>"new")}));
    $r->dynamic();
    return $r->make();
}

sub show_location {
    my ($this,$lid) = @_;
    my $section_id = $this->{section_id};
    my $loc = new Modules::ID::Location(id=>$lid);
    my $out = $ml->p($ml->a("&lt; Back to listing",{href=>$this->link(ab=>undef,lid=>undef)}));
    $out .= $loc->show();
    my $buttons = &ExSite::HTML::Button(
					label=>"Edit location",
					url=>$this->link(ab=>"edit")
					);
    $buttons .= &ExSite::HTML::Button(label=>"Add another contact",url=>$this->link(ab=>"addcon"));
    $out .= $ml->p($buttons);
    return $out;
}

sub new_location {
    my $this = shift;
    my $input = new ExSite::Input();
    my $post = $input->post();
    if (keys %$post > 0) {
	return $this->doform($post,$this->link(ab=>undef,__plaintext=>1));
    }
    my $db = $share{DB};
    $db->set_action("insert");
    $db->form(action=>$this->link());
#    $db->form()->set("highlight_rows",1);
    $db->input_record({
	table=>"location",
	record=>"_1",
	data=>{section_id=>$this->{section_id}},
	show=>["name","description","image","brochure","longitude","latitude"],
    });
    $db->form()->input(type=>"preformatted",name=>"contact_hdg",input=>$ml->h2("Contact Information"));
    $db->input_record({
	table=>"contact",
	data=>{location_id=>"_1",type=>"office"},
	show=>["type","address","city","provstate","country","pcode","phone1","fax","email","web"],
    });
    my $out = $ml->h1("New Location");
    $out .= $db->form()->make();
    return $out;
}

sub edit_location {
    my $this = shift;
    my $location = new Modules::ID::Location(id=>$this->{input}{lid});
    if (! $location->owns) {
	return $this->error("Permission denied.");
    }
    my $input = new ExSite::Input();
    my $post = $input->post();
    if (keys %$post > 0) {
	return $this->doform($post,$this->link(ab=>"view",__plaintext=>1));
    }
    my $db = $share{DB};
    $db->set_action("insert");
    $db->form(action=>$this->link());
    $db->input_record({
	table=>"location",
	data=>$location->load(),
	show=>["name","description","image","brochure","longitude","latitude"],
    });
    my $contacts = $location->contacts();
    my $ncont = $contacts->count();
    if ($ncont == 1) {
	$db->form()->input(type=>"preformatted",name=>"contact_hdg",input=>$ml->h2("Contact Information"));
    }
    while (my $contact = $contacts->next()) {
	if ($ncont > 1) {
	    $db->form()->input(type=>"preformatted",name=>$contact->getdata("type")."_hdg",input=>$ml->h3(ucfirst($contact->showdata("type"))." Contact Information:"));
	}
	$db->input_record({
	    table=>"contact",
	    data=>$contact->get(),
	    show=>["type","address","city","provstate","country","pcode","phone1","fax","email","web","privacy"],
	});
    }
    my $out = $ml->h1($location->name);
    $out .= $db->form()->make();
    return $out;
}

sub add_location_contact {
    my $this = shift;
    my $input = new ExSite::Input();
    my $post = $input->post();
    if (keys %$post > 0) {
	return $this->doform($post,$this->link(ab=>"view",__plaintext=>1));
    }
    my $db = $share{DB};
    $db->set_action("insert");
    $db->form(action=>$this->link());
    $db->input_record({
	table=>"contact",
	data=>{location_id=>$this->{input}{lid},privacy=>"public"},
	show=>["type","address","city","provstate","country","pcode","phone1","fax","email","web","privacy"],
    });
    my $loc = new Modules::ID::Location(id=>$this->{input}{lid});
    my $out = $ml->h1("Additional Contact Information for ".$loc->name());
    $out .= $db->form()->make();
    return $out;
}

# doform - process forms
# $data = post data
# $goto = redirect URL when done (plaintext format)

sub doform {
    my ($this,$data,$goto) = @_;
    if ($data->{submit_action} eq "Cancel") {
	$ml->location($goto);
	return undef;
    }
    my $db = $share{DB};
    my $out;
    $db->get_data();
    $db->{action} = $db->get_action();
    
    my $errcnt = &AtError();
    $db->do_db($db->{action});

    # report results
    if (! &AtError($errcnt)) {
	# no errors
	$ml->location($goto);
	$out .= "Done!";
    }
    else {
	# uh-oh!
	$out = $this->ml->h3("Problem:");
	$out .= $this->show_diagnostics("error","html");
	$out .= $this->ml->p($this->ml->a("Go back to form.",{href=>"javascript:history.back()"}));
    }
    return $out;
}

sub setup_querylib {
    my $this = shift;
    my $db = $share{DB};

    # contacts are associated with sections, accounts and locations

    $db->set_query(
		   "other contacts",
		   ( sql=>"select * from contact where section_id=?",
		     nparam=>1,
		     mode=>"r",
		     keys=>["contact"],
		     )
		   );

    $db->set_query(
		   "member contacts",
		   ( sql=>"select a.name,c.* from contact c,account a,member m where c.account_id=a.account_id and m.member_id=a.member_id and m.section_id=?",
		     nparam=>1,
		     mode=>"r",
		     keys=>["contact","account"],
		     )
		   );

    $db->set_query(
		   "customer contacts",
		   ( sql=>"select a.name,c.* from contact c,account a where c.account_id=a.account_id and a.member_id=0 and a.section_id=?",
		     nparam=>1,
		     mode=>"r",
		     keys=>["contact","account"],
		     )
		   );

    $db->set_query(
		   "location contacts",
		   ( sql=>"select l.location_id,l.name,c.* from contact c,location l where c.location_id=l.location_id and l.section_id=?",
		     nparam=>1,
		     mode=>"r",
		     keys=>["contact","location"],
		     )
		   );

    $db->set_query(
		   "my locations",
		   ( sql=>"select distinct l.*,c.* from contact c,location l where c.location_id=l.location_id and l.section_id=?",
		     nparam=>1,
		     mode=>"r",
		     keys=>["contact","location"],
		     )
		   );
    $db->set_query("events with registrant contacts",
		   ( 
		     sql => "select e.evt_id,e.name event,fee.evt_fee_id,fee.name fee,ed.date from evt e,evt_date ed,evt_fee fee where ed.evt_id=e.evt_id and fee.evt_id=e.evt_id and e.section_id=? order by ed.date,ed.time desc",
		     nparam => 1,
		     mode => "r",
		     keys => ["evt","evt_date","evt_fee"],
		     )
		   );
}


# publish() can be used to write out any files that will benefit from 
# being available statically

sub publish {
    my $this = shift;

    # this method will be called via publish.cgi, which has permission to
    # write files into the webserver directories.

    # It can optionally return some status messages.
    my $out;

    # your publishing code goes here.

    return $out;
}

sub unpublish {
    my $this = shift;

    # This does the opposite of publish() and removes files that this 
    # module has previously published.

    # It can optionally return some status messages.
    my $out;

    # your un-publishing code goes here.

    return $out;
}

1;
