#!/usr/bin/perl
#-----------------------------------------------------------------------
#
#   Copyright 2001-2009 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

package Modules::ID::Contact;

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::Object;

use vars qw(@ISA);
@ISA = qw(ExSite::Object);

sub my_type { "contact"; }

sub name {
    my $this = shift;
    $this->load() if (!defined $this->{data});
    return $this->{data}{info} || $this->{data}{type};
}

sub contact_name {
    my $this = shift;
    if ($this->defined) {
	if ($this->{data}{account_id}) {
	    my $acct = $share{DB}->fetch("account",$this->{data}{account_id});
	    return $acct->{name};
	}
	elsif ($this->{data}{location_id}) {
	    my $loc = $share{DB}->fetch("location",$this->{data}{location_id});
	    return $loc->{name};
	}
	else {
	    return $this->{data}{info};
	}
    }
    return undef;
}

sub showauth { 
    my ($this,$key) = @_;
    return 1;
}

sub uid {
    my ($this,$account_id) = @_;
    $account_id or $account_id = $this->getdata("account_id");
    # no explicit match criteria; match against current user
    if ($account_id) {
	my $account = $share{DB}->fetch("account",$account_id);
	return (ref $account eq "HASH") ? $account->{member_id} : undef;
    }
}

sub is_my_contact { 
    my ($this,$account_id,$location_id) = @_;
    my $acct = $this->getdata("account_id");
    my $loc = $this->getdata("location_id");

    # explicit match criteria
    if ($account_id) {
	return 1 if ($acct == $account_id);
    }
    if ($location_id) {
	return 1 if ($loc == $location_id);
    }
    return 0 if ($account_id || $location_id);

    # no explicit match criteria; match against current user
    if ($acct) {
	return 1 if ($this->uid($acct) == $share{DB}->my_uid());
    }
    if ($loc) {
	my $location = $share{DB}->fetch("location",$acct);
	if (ref $location eq "HASH") {
	    # matches my uid
	    return 1 if ($location->{member_id} == $share{DB}->my_uid());
	}
    }
    return 0;
}

sub allow {
    my $this = shift;
    my $privacy = $this->getdata("privacy") || "administrators";
    if ($privacy eq "public") {
	return 1;
    }
    elsif ($privacy eq "administrators" && $share{DB}->is_manager()) {
	return 1;
    }
    elsif ($privacy eq "members" && $share{DB}->is_member()) {
	return 1;
    }
    elsif ($this->is_my_contact()) {
	return 1;
    }
    elsif ($privacy eq "administrators") {
	# allows us to override ownership logic in some cases
	return $share{DB}->user_owns($this->{type},$this->{data});
    }
    return 0;
}

sub show {
    my ($this,%opt) = @_;
    if ($this->ok) {
	if (! $this->{data}) { $this->load(); }
	if (! $this->{data}) {
	    my $name = ucfirst $this->name;
	    return $this->error("$name does not exist.");
	}

	# privacy check
	if (! $this->allow()) {
	    $this->warn("Show contact info: permission denied.");
	    return undef;
	}

	my $input = new ExSite::Input;
	my $in = $input->combine();
	if ($in->{vcard} == $this->id) {
	    my $p = $share{Page} || $share{ML};
	    $p->cancel;
	    print "Content-type: text/x-vcard\ncontent-disposition: attachment; filename=contact.vcf\n\n";
	    print $this->vcard;
	    return;
	}

	my $ml = new ExSite::ML;
	my $c = $this->{data};
	my $out;
	if ($opt{title}) {
	    my $name;
	    if ($opt{title} != 1) {
		$name = $opt{title};
	    }
	    else {
		$name = $opt{name} ? $opt{name} : $this->contact_name();
	    }
	    $out .= $ml->strong($name).$ml->br;
	}
	my $addr = $this->show_address();

	$addr .= $ml->br;
	if ($c->{phone1}) { 
	    $addr .= $msg{TEL}.": ".$this->showdata("phone1").$ml->br;
	}
	if ($c->{phone2}) { 
	    $addr .= $msg{MOBILE}.": ".$this->showdata("phone2").$ml->br;
	}
	if ($c->{fax}) { 
	    $addr .= $msg{FAX}.": ".$this->showdata("fax").$ml->br;
	}

	$addr .= $ml->br;
	if ($c->{email}) { 
	    $addr .= $msg{"E-mail"}.": ".$this->showdata("email").$ml->br;
	}
	if ($c->{web}) { 
	    $addr .= $msg{Web}.": ".$this->showdata("web");
	}
	if ($opt{notes}&& $this->{data}{notes}) { 
	    $addr .= $ml->br;
	    $addr .= $msg{Notes}.": ".$this->showdata("notes");
	}
	if ($opt{plaintext}) {
	    $out .= $ml->p($addr);
	    return &html_to_plaintext($out);
	}
	if ((ref $opt{vcard}) =~ /URI/) {
	    $opt{vcard}->parameter("vcard",$this->id);
	    my $url = $opt{vcard}->write();
	    $addr .= $ml->br.
		$msg{"Vcard"}.": ".
		$ml->a($msg{"import to my address book"},{href=>$opt{vcard}->write(),class=>"vcard"});
	}
	$out .= $ml->p($addr);
	return $out;
    }
    return $this->error($this->{status});
}

# only show address fields, as if for a mailing label
sub show_address {
    my ($this,%opt) = @_;
    if ($this->defined) {
	my $ml = new ExSite::ML;
	my $c = $this->{data};
	my $addr;
	if ($c->{info}) { 
	    $addr .= $ml->span($this->showdata("info"),{class=>"contact_info"}).$ml->br;
	}
	if ($c->{address}) { 
	    my $address = $this->showdata("address");
	    $address =~ s/\s+$//;  # trim trailing whitespace
	    $addr .= $address.$ml->br;
	}

	my @citystate;
	if ($c->{city}) { push @citystate, $this->showdata("city"); }
	if ($c->{provstate}) { push @citystate, $this->showdata("provstate"); }
	$addr .= @citystate > 0 ? join(", ",@citystate).$ml->br : "";

	my @countrycode;
	if ($c->{country}) { push @countrycode, $this->showdata("country"); }
	if ($c->{pcode}) { push @countrycode, $this->showdata("pcode"); }
	$addr .= @countrycode > 0 ? join("  ",@countrycode).$ml->br : "";

	return $opt{plaintext} ? &html_to_plaintext($addr) : $addr;
    }
    return $this->error($this->{status});
}

# simple vcard implementation

sub vcard {
    my ($this,%opt) = @_;
    my $type = uc $this->getdata("type");
    my @vcard;
    push @vcard, "VERSION:3.0";

    # full name
#    push @vcard, "N:".$this->contact_name();  # required for strict compliance with RFC
    push @vcard, "FN:".$this->contact_name();

    # email
    my $email = $this->getdata("email");
    if ($email) {
	push @vcard, "EMAIL;TYPE=INTERNET;TYPE=$type;TYPE=PREF:$email";
    }

    #phone
    my $tel = $this->getdata("phone1");
    if ($tel) {
	push @vcard, "TEL;TYPE=$type;TYPE=PREF:$tel";
    }
    $tel = $this->getdata("phone2");
    if ($tel) {
	push @vcard, "TEL;TYPE=CELL:$tel";
    }
    $tel = $this->getdata("fax");
    if ($tel) {
	push @vcard, "TEL;TYPE=FAX:$tel";
    }

    # address
    my $addr = $this->show_address(plaintext=>1);
    $addr =~ s/,\s/;/g;
    $addr =~ s/\n/;/g;
    push @vcard, "ADR;TYPE=type:$addr";

    # url
    my $url = $this->getdata("web");
    if ($url) {
	push @vcard, "URL:$url";
    }

    # note
    my $note = $this->getdata("notes");
    if ($note) {
	$note =~ s/[\r\n]/ /g;
	push @vcard, "NOTE:$note";
    }

    return "BEGIN:VCARD\n".
	join("\n",@vcard).
	"\nEND:VCARD\n";
}

# compare - compare two contact datahashes to determine how similar they
# are (0=totally different, 1=same). This can be used to determine whether
# we already have a similar contact record on file.

sub compare {
    my ($this,%contact) = @_;
    return &ExSite::Misc::compare($this->{data},\%contact);
}

# misc DB ops

sub validate {
    my $this = shift;
    return ($msg{"no data"}) if (ref $this->{data} ne "HASH");
    foreach my $k (keys %{$this->{data}}) {
	if ($k =~ /^country|provstate$/ && $this->{data}{$k} =~ /--|==/) {
	    # user selected a divider setting, which has no meaning
	    # just delete the data, don't throw an error
	    delete $this->{data}{$k};
	}
    }
    my @err = $this->SUPER::validate();
    return @err;
}

sub make { 
    my ($this,%opt) = @_;
    my $ml = &get_obj("ML");
    my $out;
    if ($this->approve("insert")) {
	my $in = new ExSite::Input();
	my $post = $in->post();
	if (keys %$post > 0) {
	    # no action provided
	    $out .= $this->do_make();
	}
	else {
	    $out .= $this->contact_form(%opt);
	    if ($opt{help}) {
		$out .= $opt{help};
	    }
	}
    }
    return $out;
}

sub edit { 
    my ($this,%opt) = @_;
    my $ml = &get_obj("ML");
    my $out;
    if ($this->loaded) {
	if ($this->approve("update")) {
	    my $in = new ExSite::Input();
	    my $post = $in->post();
	    if (keys %$post > 0) {
		# no action provided
		$out .= $this->do_edit();
	    }
	    else {
		$out .= $this->contact_form(%opt);
		if ($opt{help}) {
		    $out .= $opt{help};
		}
	    }
	}
    }
    return $out;
}

sub do_make { 
    my ($this,%opt) = @_;
    my $ml = &get_obj("ML");
    my $out;
    if ($this->approve("insert")) {
	my $in = new ExSite::Input();
	my $post = $in->post();
	my %data = $share{DB}->parse_parts(%$post);
	if (keys %data > 0) {
	    delete $data{action};
	    # attribute the record
	    if ($opt{location_id}) {
		$data{location_id} = $opt{location_id};
	    }
	    elsif ($opt{account_id}) {
		$data{account_id} = $opt{account_id};
	    }
	    else {
		# uncertain ownership - try attaching to the current user's
		# account
		my @acct = $share{DB}->fetch_child("account","member",$share{DB}->my_uid());
		if (@acct > 0) {
		    $data{account_id} = $acct[-1]{account_id};
		}
		else {
		    $this->warn("inserting unattributed contact record");
		}
	    }
	    my $c = new Modules::ID::Contact(data=>\%data);
	    $c->set("dirty",1);
	    my @match = $share{DB}->fetch_match("contact",{type=>$data{type},privacy=>$data{privacy},account_id=>$data{account_id}});
	    if (@match > 0) {
		$out .= $this->error("We already have a record on file with this type and privacy setting.  Please update that record instead of adding a new one.");
	    }
	    elsif ($c->save) {
		$out .= $ml->p("Contact information saved.");
	    }
	    else {
		$out .= $c->show_diagnostics("error","html");
	    }
	    return $out;
	}
	else {
	    $out .= $this->error("No contact data provided.");
	}
    }
    else {
	$out .= $this->error("Permission denied.");
    }
    return $out;
}

sub do_edit { 
    my ($this,%opt) = @_;
    my $ml = &get_obj("ML");
    my $out;
    my $in = new ExSite::Input();
    my $post = $in->post();
    my %data = $share{DB}->parse_parts(%$post);
    if (keys %data > 0) {
	delete $data{action};
	$this->setup(id=>$data{contact_id});
	if ($this->defined) {
	    foreach my $key (keys %data) {
		$this->setdata($key,$data{$key});
	    }
	}
	if ($this->approve("update")) {
	    $this->set("dirty",1);
	    if ($this->save) {
		$out .= $ml->p("Contact information saved.");
	    }
	    else {
		$out .= $share{DB}->show_diagnostics("error","html");
	    }
	}
	else {
	    $out .= $this->error("Permission denied.");
	}
    }
    return $out;
}

sub contact_form { 
    my ($this,%opt) = @_;
    my $ml = &get_obj("ML");
    my $db = $share{DB};
    my $admin_separator = ($share{DB}->level > 1) ? 
	$ml->hr().$ml->h4($msg{"Administrator-only fields"}) : 
	"";
    $db->form()->template(
			  $ml->div(
				   $ml->table(
					      $ml->tr(
						      $ml->td("[[type:prompt]]".$ml->br."[[type:input]]").
						      $ml->td("[[privacy:prompt]] (determines who can view this information)".$ml->br."[[privacy:input]]")
						      )
					      ).
				   $ml->table(
					      $ml->tr(
						      $ml->td("[[address:prompt]]".$ml->br."[[address:input]]",{colspan=>2})
						      ).
					      $ml->tr(
						      $ml->td("[[city:prompt]]".$ml->br."[[city:input]]").
						      $ml->td("[[provstate:prompt]]".$ml->br."[[provstate:input]]")
						      ).
					      $ml->tr(
						      $ml->td("[[country:prompt]]".$ml->br."[[country:input]]").
						      $ml->td("[[pcode:prompt]]".$ml->br."[[pcode:input]]")
						      )
					      ).
				   $ml->table(
					      $ml->tr(
						      $ml->td("[[phone1:prompt]]".$ml->br."[[phone1:input]]",{width=>"33%"}).
						      $ml->td("[[phone2:prompt]]".$ml->br."[[phone2:input]]",{width=>"33%"}).
						      $ml->td("[[fax:prompt]]".$ml->br."[[fax:input]]",{width=>"33%"})
						      ).
					      $ml->tr(
						      $ml->td("[[email:prompt]]".$ml->br."[[email:input]]",{colspan=>3})
						      ).
					      $ml->tr(
						      $ml->td("[[web:prompt]]".$ml->br."[[web:input]]",{colspan=>3})
						      )
					      ),
				   {class=>"ContactForm"}
				   ).
			  $admin_separator
			  );
    return $db->make(
		     table=>"contact",
		     record=>$this->id,
		     simplename=>1,
		     %opt,
		     );
}

sub contact_form_old { 
    my ($this,$action,$data) = @_;
    my $ml = &get_obj("ML");
    my $f = new ExSite::FormBuilder(
				    action=>$action,
				    data=>$data,
				    method=>"post"
				    );
    $f->input(name=>"contact_id",type=>"hidden");
    $f->input(prompt=>"Contact type",
	      name=>"type",
	      type=>"select",
	      options=>$share{DB}{map}->regexp("list:contact_type"));
    $f->input(prompt=>"Privacy",
	      name=>"privacy",
	      type=>"select",
	      options=>$share{DB}{map}->regexp("list:contact_privacy"));
    $f->input(prompt=>"Street/Mailing Address",
	      name=>"address",
	      type=>"textarea",
	      size=>120);
    $f->input(prompt=>"City",
	      name=>"city",
	      type=>"text",
	      size=>40);
    $f->input(prompt=>"State/Province",
	      name=>"provstate",
	      type=>"select",
	      options=>$share{DB}{map}->regexp($share{DB}{map}->get_column_attr("contact","provstate","datatype")));
    $f->input(prompt=>"Country",
	      name=>"country",
	      type=>"select",
	      options=>$share{DB}{map}->regexp($share{DB}{map}->get_column_attr("contact","country","datatype")));
    $f->input(prompt=>"Postal/Zip Code",
	      name=>"pcode",
	      type=>"text",
	      size=>10);
    $f->input(prompt=>"Telephone",
	      name=>"phone1",
	      type=>"text",
	      size=>15);
    $f->input(prompt=>"Mobile",
	      name=>"phone2",
	      type=>"text",
	      size=>15);
    $f->input(prompt=>"Fax",
	      name=>"fax",
	      type=>"text",
	      size=>15);
    $f->input(prompt=>"E-mail",
	      name=>"email",
	      type=>"text",
	      size=>40);
    $f->input(prompt=>"Website",
	      name=>"web",
	      type=>"text",
	      size=>40);

    my $admin_separator = ($share{DB}->level > 1) ? 
	$ml->hr().$ml->h4($msg{"Administrator-only fields"}) : 
	"";
    $f->template(
		 $ml->table(
			    $ml->tr(
				    $ml->td("[[type:prompt]]".$ml->br."[[type:input]]").
				    $ml->td("[[privacy:prompt]] (determines who can view this information)".$ml->br."[[privacy:input]]",{colspan=>2})
				    ).
			    $ml->tr(
				    $ml->td("[[address:prompt]]".$ml->br."[[address:input]]",{colspan=>3})
				    ).
			    $ml->tr(
				    $ml->td("[[city:prompt]]".$ml->br."[[city:input]]",{colspan=>2}).
				    $ml->td("[[provstate:prompt]]".$ml->br."[[provstate:input]]")
				    ).
			    $ml->tr(
				    $ml->td("[[country:prompt]]".$ml->br."[[country:input]]",{colspan=>2}).
				    $ml->td("[[pcode:prompt]]".$ml->br."[[pcode:input]]")
				    ).
			    $ml->tr(
				    $ml->td("[[phone1:prompt]]".$ml->br."[[phone1:input]]",{width=>"33%"}).
				    $ml->td("[[phone2:prompt]]".$ml->br."[[phone2:input]]",{width=>"33%"}).
				    $ml->td("[[fax:prompt]]".$ml->br."[[fax:input]]",{width=>"33%"})
				    ).
			    $ml->tr(
				    $ml->td("[[email:prompt]]".$ml->br."[[email:input]]",{colspan=>3})
				    ).
			    $ml->tr(
				    $ml->td("[[web:prompt]]".$ml->br."[[web:input]]",{colspan=>3})
				    )
			    ).
		 $admin_separator
		 );

    return $f->make;
}

1;

