#!/usr/bin/perl
#-----------------------------------------------------------------------
#
#   Copyright 2001-2010 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

package Modules::ID::ContactList;

use strict;
use ExSite::Config;
use ExSite::Misc;
use ExSite::ObjectList;

use vars qw(@ISA);
@ISA = qw(ExSite::ObjectList);

sub my_type { "contact"; }

sub section_id {
    my $this = shift;
    my $s = $share{DB}->this_site();
    return $s ? $s->{section_id} : 0;
}

### TODO:
#
# select customers who bought a particular item
# select customers who bought from a particular account code
# select customers who made purchases in a time window
# select customers - accounts receivable
#
# merge dupe emails
# merge dupe addresses

sub remove_unsubscribed {
    my $this = shift;
    my @newlist;
    $this->reset();
    while (my $c = $this->next()) {
	if (exists $c->{member_subscribe} && $c->{member_subscribe} =~ /n/i) {
	    # member-level unsubscribe - affects all contact records
	    next;
	}
	if ($c->{subscribe} =~ /n/i) {
	    # contact-level unsubscribe
	    next;
	}
	push @newlist, $c->get();
    }
    $this->{list} = \@newlist;
}

sub remove_dupe_emails {
    my $this = shift;
    $this->reset();
    my %email;
    while (my $c = $this->next()) {
	if ($c->{email}) {
	    if (exists $email{$c->{email}}) {
		push @{$email{$c->{email}}},$c->get();
	    }
	    else {
		$email{$c->{email}} = [$c->get()];
	    }
	}
    }
    my @newlist;
    foreach my $email (keys %email) {
	push @newlist, $this->best_of($email{$email});
    }
    $this->{list} = \@newlist;
}

sub remove_dupe_addresses {
    my $this = shift;
    $this->reset();
    my %addr;
    while (my $c = $this->next()) {
	# convert addresses to simple text form to eliminate variances in
	# spacing, punctuation, capitalization
	my $addr = uc "$c->{address} $c->{city} $c->{provstate} $c->{pcode} $c->{country}";
	$addr =~ s/[^\w]//g;
	# would also be nice to detect minor spelling differences, 
	# abbreviations, etc...
	if (exists $addr{$addr}) {
	    push @{$addr{$addr}},$c->get();
	}
	else {
	    $addr{$addr} = [$c->get()];
	}
    }
    my @newlist;
    foreach my $addr (keys %addr) {
	push @newlist, $this->best_of($addr{$addr});
    }
    $this->{list} = \@newlist;
}

sub best_of_account {
    my $this = shift;
    $this->reset();
    my %acct;
    my @newlist;
    while (my $c = $this->next()) {
	if ($c->{account_id}) {
	    if (exists $acct{$c->{account_id}}) {
		push @{$acct{$c->{account_id}}},$c->get();
	    }
	    else {
		$acct{$c->{account_id}} = [$c->get()];
	    }
	}
	else {
	    push @newlist, $c->get();
	}
    }
    foreach my $acct (keys %acct) {
	push @newlist, $this->best_of($acct{$acct});
    }
    $this->{list} = \@newlist;
}

sub best_of_location {
    my $this = shift;
    $this->reset();
    my %loc;
    my @newlist;
    while (my $c = $this->next()) {
	if ($c->{location_id}) {
	    if (exists $loc{$c->{location_id}}) {
		push @{$loc{$c->{location_id}}},$c->get();
	    }
	    else {
		$loc{$c->{location_id}} = [$c->get()];
	    }
	}
	else {
	    push @newlist, $c->get();
	}
    }
    foreach my $loc (keys %loc) {
	push @newlist, $this->best_of($loc{$loc});
    }
    $this->{list} = \@newlist;
}

sub best_of {
    my ($this,$list) = @_;
    if (scalar @$list == 1) {
	return $list->[0];
    }

    # for now, the 'best' contact record is defined as the one with the
    # most defined fields, but this should probably have some configurable
    # heuristics, eg.
    # - preferred types
    # - age of record
    # could also merge records with different fields set

    my $ibest;
    my $cmax = -1;
    for (my $i=0; $i < scalar @$list; $i++) {
	my $count = 0;
	foreach my $key (keys %{$list->[$i]}) {
	    if ($list->[$i]{$key}) { $count++; }
	}
	if ($count > $cmax) { $ibest = $i; }
    }
    return $list->[$ibest];
}

sub where {
    my ($this, $match) = @_;
    my (@wlist,@plist,$value);
    while (my ($key, $val) = each(%$match)) {
	# sanity check on keys
	if ($key !~ /^[\w\.]+$/) {
	    $this->error("invalid match column: $key");
	    return undef;
	}
	my $op = "=";
	if (ref $val) {
	    # values passed by reference are SQL literals
	    if ($$val =~ /^null$/i) { 
		$op = "IS"; 
	    }
	    $value = $$val;
	}
	elsif (! $val) {
	    next;
	}
	else {
	    # normal values are user data
	    if ($val =~ /%/) {
		if ($val =~ /(?<!\\)%/) {  # any occurence of % not preceeded by \
		    $op = "LIKE";
		}
		$val =~ s/\\%/%/g;
	    }
	    $value = "?";
	    push @plist, $val;
	}
	push @wlist, "$key $op $value";
    }
    return \@wlist,\@plist;
}

sub load_db {
    my ($this,$id) = @_;
    my $out;
    my $clist = new ExSite::Object("contact_list",$id);
    return if (! $clist->exists);
    my $ctype= $clist->getdata("type");
    my %in = &DecodeString($clist->getdata("parameters"));
    my %match;
    my $method;
    if ($in{provstate} && $in{provstate} !~ /==/) {
	$match{provstate} = $in{provstate};
    }
    if ($in{country}) {
	$match{country} = $in{country};
    }
    if ($in{type}) {
	$match{"c.type"} = $in{type};
    }
    if ($ctype eq "member") {
	if ($in{usertype} || $in{status}) {
	    $method = "load_membership";
	    if ($in{usertype}) {
		$match{'member_type'} = $in{usertype};
	    }
	    if ($in{status}) {
		$match{status} = $in{status};
	    }
	}
	else {
	    $method = "load_members";
	}
    }
    elsif ($ctype eq "customer") {
	$method = "load_customers";
    }
    elsif ($ctype eq "registrant") {
	my ($evt,$fee) = split /-/,$in{evtfee};
	$match{evt_id} = $evt;
	if ($fee) { $match{evt_fee_id} = $fee; }
	if ($in{status}) {
	    $match{'r.status'} = $in{status};
	}
	$method = "load_registrants";
    }
    elsif ($ctype eq "location") {
	$method = "load_locations";
    }
    elsif ($ctype eq "other") {
	if ($in{group}) {
	    $match{category} = $in{group};
	}
	$method = "load_other";
    }
    $this->$method(\%match);
    if ($in{unsubscribe} eq "exclude") {
	$this->remove_unsubscribed();
    }
    if ($in{dupe} =~ /best/) {
	if ($in{ctype} eq "user") {
	    $this->best_of_account();
	}
	elsif ($in{ctype} eq "customer") {
	    $this->best_of_account();
	}
	elsif ($in{ctype} eq "location") {
	    $this->best_of_location();
	}
	elsif ($in{ctype} eq "other") {
	    $this->remove_dupe_emails();
	    $this->remove_dupe_addresses();
	}
    }
}

# members : simple attachment of members to their contact records

sub load_members {
    my ($this,$match) = @_;
    my $sql = "select a.name,m.login,m.password,c.* from contact c,account a,member m where ";
    my @wlist = ("c.account_id=a.account_id", 
		 "m.member_id=a.member_id",
		 "m.section_id=?");
    my @param = ($this->section_id);
    my ($xtra_w,$xtra_p) = $this->where($match);
    if (ref $xtra_w eq "ARRAY") {
	push @wlist, @$xtra_w;
    }
    $sql .= join(" and ",@wlist);
    if (ref $xtra_p eq "ARRAY") {
	push @param, @$xtra_p;
    }
    my @c = $share{DB}->custom_query($sql,@param);
    $this->{list} = \@c;
}

# membership : also includes member_status, if that is installed

sub load_membership {
    my ($this,$match) = @_;
    my $sql = "select a.name,m.type,m.first_name,m.middle_name,m.last_name,m.login,m.password,m.gender,m.subscribe member_subscribe,m.type member_type,c.*,(select status from member_status where member_status.member_id=m.member_id order by member_status_id desc limit 1) status from contact c,account a,member m where ";
    my @wlist = ("c.account_id=a.account_id", 
		 "m.member_id=a.member_id",
		 "m.section_id=?");
    my @param = ($this->section_id);
    my %having;
    if (exists $match->{status}) {
	$having{status} = $match->{status};
	delete $match->{status};
    }
    if (exists $match->{member_type}) {
	$having{member_type} = $match->{member_type};
	delete $match->{member_type};
    }
    my ($xtra_w,$xtra_p) = $this->where($match);
    if (ref $xtra_w eq "ARRAY") {
	push @wlist, @$xtra_w;
    }
    $sql .= join(" and ",@wlist);
    if (ref $xtra_p eq "ARRAY") {
	push @param, @$xtra_p;
    }
    if (scalar keys %having > 0) {
	my ($xtra_w,$xtra_p) = $this->where(\%having);
	if (ref $xtra_w eq "ARRAY") {
	    $sql .= " having ".join(" and ",@$xtra_w);
	    push @param, @$xtra_p;
	}
    }
    my @c = $share{DB}->custom_query($sql,@param);
    $this->{list} = \@c;
}

# membership_meta : same as above, but also allows filtering on one 
# meta data column

sub load_membership_meta {
    my ($this,$match) = @_;
    my $sql = "select a.name,m.type,m.first_name,m.middle_name,m.last_name,m.login,m.password,m.gender,a.value meta,c.*,(select status from member_status where member_status.member_id=m.member_id order by member_status_id desc limit 1) status from contact c,account a,member m,member_attribute a where ";
    my @wlist = ("c.account_id=a.account_id", 
		 "m.member_id=a.member_id",
		 "a.member_id=m.member_id",
		 "m.section_id=?");
    my @param = ($this->section_id);
    my $status;
    if (exists $match->{status}) {
	$status = $match->{status};
	delete $match->{status};
    }
    foreach my $key (keys %$match) {
	if ($key =~ /^meta_/) {
	    my $metaname = $key;
	    $metaname =~ s/^meta_//;
	    push @wlist, "a.name=?";
	    push @param, $metaname;
	    last;  # only one meta column
	}
    }
    my ($xtra_w,$xtra_p) = $this->where($match);
    if (ref $xtra_w eq "ARRAY") {
	push @wlist, @$xtra_w;
    }
    $sql .= join(" and ",@wlist);
    if (ref $xtra_p eq "ARRAY") {
	push @param, @$xtra_p;
    }
    if ($status) {
	$sql .= " having status ?";
	push @param, $status;
    }
    my @c = $share{DB}->custom_query($sql,@param);
    $this->{list} = \@c;
}

# registrants : contact records that can be tied to an evt_reg record

sub load_registrants {
    my ($this,$match) = @_;
    my ($evt_id,$fee_id);
    if ($match->{evt_id}) {
	$evt_id = $match->{evt_id};
	delete $match->{evt_id};
    }
    else {
	$this->error("Cannot build a registrant contact list without an event ID.");
	return;
    }
    if ($match->{evt_fee_id}) {
	$fee_id = $match->{evt_fee_id};
	delete $match->{evt_fee_id};
    }
    my $sql = "select r.first_name,r.last_name,a.name name, c.* from contact c, account a, evt_reg r, evt_fee f where ";
    my @wlist = ("c.account_id=a.account_id",
		 "r.account_id=a.account_id",
		 "r.evt_fee_id=f.evt_fee_id",
		 "f.evt_id=?");
    my @param = ($evt_id);
    if ($fee_id) {
	push @wlist, "f.evt_fee_id=?";
	push @param, $fee_id;
    }
    if (! $match->{'r.status'}) {
	push @wlist, "r.status='active'",
    }
    my ($xtra_w,$xtra_p) = $this->where($match);
    if (ref $xtra_w eq "ARRAY") {
	push @wlist, @$xtra_w;
    }
    $sql .= join(" and ",@wlist);
    if (ref $xtra_p eq "ARRAY") {
	push @param, @$xtra_p;
    }
    my @c = $share{DB}->custom_query($sql,@param);
    $this->{list} = \@c;

    ### FIXME: now add 'other' contact records 
}

# customers : have accounts, but no member record

sub load_customers {
    my ($this,$match) = @_;
    my $sql = "select a.name name,c.* from contact c,account a,member m where ";
    my @wlist = ("c.account_id=a.account_id",
		 "a.member_id=0",
		 "a.section_id=?");
    my @param = ($this->section_id);
    my ($xtra_w,$xtra_p) = $this->where($match);
    if (ref $xtra_w eq "ARRAY") {
	push @wlist, @$xtra_w;
    }
    $sql .= join(" and ",@wlist);
    if (ref $xtra_p eq "ARRAY") {
	push @param, @$xtra_p;
    }
    my @c = $share{DB}->custom_query($sql,@param);
    $this->{list} = \@c;
}

# locations : contact records tied directly to a location record

sub load_locations {
    my ($this,$match) = @_;
    my $sql = "select l.location_id,l.name,c.* from contact c,location l where ";
    my @wlist = ("c.location_id=l.location_id", 
		 "l.section_id=?");
    my @param = ($this->section_id);
    my ($xtra_w,$xtra_p) = $this->where($match);
    if (ref $xtra_w eq "ARRAY") {
	push @wlist, @$xtra_w;
    }
    $sql .= join(" and ",@wlist);
    if (ref $xtra_p eq "ARRAY") {
	push @param, @$xtra_p;
    }
    my @c = $share{DB}->custom_query($sql,@param);
    $this->{list} = \@c;
}

# other : contact records not matching any of the above

sub load_other {
    my ($this,$match) = @_;
    my $sql = "select c.* from contact c where ";
    my @wlist = ("account_id=0",
		 "location_id=0",
		 "section_id=?");
    my @param = ($this->section_id);
    my ($xtra_w,$xtra_p) = $this->where($match);
    if (ref $xtra_w eq "ARRAY") {
	push @wlist, @$xtra_w;
    }
    $sql .= join(" and ",@wlist);
    if (ref $xtra_p eq "ARRAY") {
	push @param, @$xtra_p;
    }
    my @c = $share{DB}->custom_query($sql,@param);
    $this->{list} = \@c;
}

1;

