#!/usr/bin/perl
#-----------------------------------------------------------------------
#
#   Copyright 2001-2007 Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

package Modules::Event::Event;

use strict;
use ExSite::Config;
use ExSite::Base;
use ExSite::Form;
use ExSite::Misc;
use ExSite::ML;
use ExSite::Object;
use ExSite::ObjectList;
use Modules::ID::Location;

use vars qw(@ISA $ml %datetype);
@ISA = qw(ExSite::Object);

%datetype = (
	     start => "Event begins",
	     end => "End of event",
	     "open registration" => "Open for registration",
	     "close registration" => "Registration closes",
	     "open early registration" => "Open for early-bird registration",
	     "close early registration" => "Early-bird registration closes",
	     "open late registration" => "Accepting late registrations",
	     "close late registration" => "End of late registration period",
	     );

sub my_type { return "evt"; }    # generic object has no preset type

sub name {
    my $this = shift;
    if ($this->defined()) {
	return $this->getdata("name");
    }
    return "n/a";
}

sub event_type {
    my $this = shift;
    my $type = $this->getdata("type");
    return $type if ($type);
    return $this->getdata("parent") ? "activity" : "event";
}

sub is_active {
    my $this = shift;
    return $this->getdata("status") =~ /^(active|hidden)$/;
}

sub has_registration {
    my $this = shift;
    my $fees = $this->get_fees();
    if ($fees && $fees->count > 0) {
	foreach my $fee ($fees->next) {
	    return 1 if ($fee->getdata("status") ne "inactive");
	}
	return 0;  # means there are fees, but none are active
    }
    return undef;  # means there are no fees
}

sub sold_out {
    my $this = shift;
    my $reglimit = $this->getdata("reglimit");
    if ($reglimit && $this->has_registration) {
	my $fees = $this->get_fees();
	my $total = 0;
	while (my $fee = $fees->next()) {
	    $total += $fee->count_registrations();
	}
	return ($total >= $reglimit);
    }
    return undef;
}

sub location {
    my ($this,$type) = @_;
    my @location;
    if ($this->get_children()) {
	if ($this->{children}{evt_location}) {
	    if ($this->{children}{evt_location}->count > 0) {
		while (my $eloc = $this->{children}{evt_location}->next()) {
		    my $loc = $share{DB}->fetch("location",$eloc->getdata("location_id"));
		    push @location, $loc->{name};
		}
	    }
	}
    }
    return join "; ", @location;
}

# compound events

sub parent_event {
    my $this = shift;
    my $parent = $this->getdata("parent");
    if ($parent) {
	my $evt = new Modules::Event::Event(id=>$parent);
	return $evt;
    }
    return undef;
}

sub main_event {
    my $this = shift;
    my $evt = $this->parent_event();
    if ($evt) {
	while (my $parent = $evt->parent_event()) {
	    $evt = $parent;
	}
    }
    return $evt;
}

sub main_registration_event {
    my $this = shift;
    my $regevt;
    my $evt = $this;
    if ($evt->has_registration) {
	$regevt = $evt;
    }
    while (my $parent = $evt->parent_event()) {
	$evt = $parent;
	if ($evt->has_registration) {
	    $regevt = $evt;
	}
    }
    return $regevt;
}

sub owner {
    my $this = shift;
    my $owner;
    if ($this->getdata("owner")) {
	$owner = $share{DB}->fetch($config{auth}{user_table},$this->getdata("owner"));
    }
    if (! $owner) {
	$owner = $share{DB}->fetch_owner();
    }
    return $owner;
}

# display the event
# Warning - no access controls - application must provide appropriate 
# access controls.

sub show {
    my ($this,%opt) = @_;
    if ($opt{brief}) {
	return $this->show_brief();
    }
    $ml = new ExSite::ML;
    $this->setup_querylib();
    my $status = $this->getdata("status");
    my $out;
    if ($status eq "inactive" && ! $share{DB}->is_manager()) {
	return $this->error($msg{"Permission denied."});
    }
    if ($this->getdata("privacy") =~ /members/ && $share{DB}->level < 1) {
	return $ml->p($msg{"To view this event, you must be logged in."});
    }

    $out .= $this->getdata("parent") ?
	$ml->h2($this->showdata("name")) :
	$ml->h1($this->showdata("name"));

    # show simple date at top (full schedule at bottom)
    my $simple_date = $this->show_date_range;
    $out .= $ml->p($simple_date,{class=>"EventDate"});

    if ($status =~ /canceled/) {
	$out .= $ml->div($msg{"This event has been canceled.  Event information is shown for archival purposes only."},
			 { style=>"border:1px dashed #c00; color:#c00; background-color:#fcc; font-weight:bold; font-size:120%;" });
    }

    # warning: we are using getdata, which means we pass through unsanitized 
    # data. We must take steps to sanitize this data at the time of data
    # entry if it comes from untrusted users.
    $out .= $ml->p($this->getdata("description"));

    if (! exists $opt{schedule} || $opt{schedule}) {
	# only show schedule if there is more than 1 date on it, since
	# we already show the date range at the top
	my $dates = $this->get_dates();
	if ($dates->count > 1) {
	    $out .= $this->show_schedule(%opt);
	}
    }
    if ($this->getdata("parent")) {
	my $pid = $this->getdata("parent");
	my $evt = new Modules::Event::Event(id=>$pid);
	my $name = $evt->name;
	if ($opt{url}) {
	    $opt{url}->reset;
	    $opt{url}->query(evt=>$pid);
	    $name = $ml->a($name,{href=>$opt{url}->write()});
	}
	my $in = new ExSite::Input;
	my $indata = $in->get();
	if ($indata->{evt} && $indata->{evt} != $pid) {
	    $out .= $ml->p(&substitute($msg{"This [[type]] is part of [[event]]."},{type=>$evt->event_type,event=>$name}));
	}
    }
    if (! exists $opt{location} || $opt{location}) {
	$out .= $this->show_locations(%opt);
    }
    if ($opt{show_activities}) {
	my @activity = $share{DB}->get_query("activities",$this->id);
	my $acts;
	if (@activity > 0) {
	    foreach my $act (@activity) {
		my $evt = new Modules::Event::Event(data=>$act);
		my $act .= $evt->show_brief(%opt);
		if ($opt{url}) {
		    $opt{url}->reset;
		    $opt{url}->parameter("evt",$evt->id);
		    $act .= $ml->p($ml->a($msg{"More information..."},{href=>$opt{url}->write()}));
		}
		$acts .= $ml->li($act);
	    }
	}
	$out .= $ml->ul($acts) if $acts;
    }
    if (! exists $opt{fee} || $opt{fee}) {
	if ($this->has_registration()) {
	    $out .= $this->getdata("parent") ?
		$ml->h3($msg{Registration}) :
		$ml->h2($msg{Registration});
	    $opt{url}->reset if $opt{url};
	    $out .= $this->show_fees(%opt);
	}
    }

    return $ml->div($out,{class=>"Event"});
}

# abbreviated view for pop-up event display

sub show_brief {
    my ($this,%opt) = @_;
    $ml = new ExSite::ML;
    $this->setup_querylib();
    my $status = $this->getdata("status");
    my $out;

    ### screen events based on status

    my $out = $ml->h3($this->showdata("name"));
    if ($status =~ /canceled/) {
	$out .= $ml->p($msg{"CANCELED"});
    }
    $out .= $ml->p($this->show_date_range());
    $out .= $ml->p($this->showdata("description"));
    return $ml->div($out,{class=>"Event"});
}

sub summary {
    my ($this,%opt) = @_;
    my $out;
    $ml = new ExSite::ML;
    if ($this->is_active) {
	$out = $opt{url} ?
	    $ml->strong($ml->a($this->name,{href=>$opt{url}})) :
	    $ml->strong($this->name);
	$out .= $ml->br;
	$out .= $this->date();
	$out = $ml->p($out);
    }
    return $out;
}

sub get_schedule {
    my ($this,%opt) = @_;
    my $out;
    my @date;
    if ($this->loaded()) {
	my $times = $this->{children}{evt_date};
	$times->reset();
	my $ml = &get_obj("ML");
	while (my $t = $times->next()) {
	    my $sortdate = $t->getTime->write("sql_datetime");
	    my $date = $t->show;
	    my $descr = $t->getdata("description") || $this->name();
	    if ($opt{url}) {
		$opt{url}->parameter("evt",$t->getdata("evt_id"));
		$descr = $ml->a($descr,{href=>$opt{url}->write});
	    }
	    push @date, { sort=>$sortdate, date=>$date, description=>$descr };
	}

	if ($opt{show_activities}) {
	    my @activity = $share{DB}->get_query("activities",$this->id);
	    if (@activity > 0) {
		my $activities = new ExSite::ObjectList(type=>"evt",list=>\@activity);
		while (my $evt = $activities->next()) {
		    push @date, $evt->get_schedule(%opt);
		}
	    }
	}
    }
    return wantarray ? @date : \@date;
}

sub show_schedule {
    my ($this,%opt) = @_;
    my $out;
    my @date = sort { $a->{sort} cmp $b->{sort} } $this->get_schedule(%opt);
    my $ndate = scalar @date;
    if ($ndate == 1) {
	$out .= $ml->h2($msg{Date});
	$out .= $ml->p($date[0]{date});
    }
    elsif ($ndate > 1) {
	$out .= $ml->h2($msg{Schedule});
	$out .= $this->show_schedule_table(@date);
    }
    return $out;
}

sub show_schedule_table {
    my ($this,@date) = @_;
    my $r = new ExSite::ReportBuilder(
				      title=>"Schedule",
				      headers=>["Event","Date"]
				      );
    scalar @date > 0 or @date = $this->get_schedule();
    foreach my $date (@date) {
	$r->push($date->{description},$date->{date});
    }
    return $r->make;
}

sub show_locations {
    my ($this,%opt) = @_;
    my $out;
    if ($this->exists()) {
	my $nloc = $this->{children}{evt_location}->count();
	if ($nloc > 0) {
	    my @list;
	    while (my $loc = $this->{children}{evt_location}->next()) {
		next if ($loc->getdata("status") eq "inactive");
		my $venue = new Modules::ID::Location(id=>$loc->getdata("location_id"));
		push @list, $venue->show(%opt);
	    }
	    if (@list > 1) { 
		$out .= $ml->h2("Locations"); 
		$out .= $ml->ul(\@list);
	    }
	    elsif (@list == 1) { 
		$out .= $list[0];
	    }
	}
    }
    return $out;
}

# simple vcalendar implementation
# - each date is output as a separate vevent in the vcalendar object

sub vcalendar {
    my ($this,%opt) = @_;
    my @vcal = (
		"VERSION:2.0",
		"PRODID:-//Exware Solutions Inc.//ExSite v$ExSite::Config::VERSION//EN",
		);

    my $now = new ExSite::Time();
    my $dates = $this->get_dates();
    my $owner = $this->owner;
    while (my $date = $dates->next()) {
	push @vcal, "BEGIN:VEVENT";
	push @vcal, "UID:".$owner->{$config{auth}{user_email_column}};
	push @vcal, "DTSTAMP:".$now->write("iso_basic");

	### owner may not be organizer!
	push @vcal, "ORGANIZER;CN=".$share{DB}->user_name($owner).
	    ":MAILTO:".$owner->{$config{auth}{user_email_column}};

	my $t1 = $date->get_start();
	my $t2 = $date->get_end();
	
	if ($t1->has_time) {
	    push @vcal, "DTSTART:".$t1->write("iso_basic");
	    if ($t1->diff($t2) > 0) {
		push @vcal, "DTEND:".$t2->write("iso_basic");
	    }
	}
	else {
	    push @vcal, "DTSTART:".$t1->write("%YYYY%MM%DD");
	    if ($t1->diff($t2) > 0) {
		$t2->add(1,"day");  # because vcal dates are non-inclusive
		push @vcal, "DTEND:".$t2->write("%YYYY%MM%DD");
	    }
	}

	# include date description in summary
	my $summary = $this->name;
	my $date_descr = $date->getdata("description");
	if ($date_descr) {
	    $summary .= " - ".$date_descr;
	}
	push @vcal, "SUMMARY:".$summary;
	
	# cancellations
	if ($this->getdata("status") =~ /cancel/) {
	    push @vcal, "METHOD:CANCEL";
	    push @vcal, "STATUS:CANCELLED";
	}

	push @vcal, "END:VEVENT";
    }

    return "BEGIN:VCALENDAR\n".
	join("\n",@vcal).
	"\nEND:VCALENDAR\n";
}

#=== sub events

sub activities {
    my $this = shift;
    if ($this->loaded()) {
	if ($this->{children}{evt}) {
	    return $this->{children}{evt};
	}
    }
    return undef;
}

# event_group - all related events, as an ObjectList 

sub event_group {
    my $this = shift;
    my $events = new ExSite::ObjectList(type=>"evt");
    my $mainevt = $this->main_event() || $this;
    $events->push($mainevt->get());
    while (my $evt = $events->next) {
	my $activities = $evt->activities();
	if ($activities) {
	    while (my $act = $activities->next) {
		$events->push($act->get());
	    }
	}
    }
    return $events;
}

# ...as a Tree

sub event_group_tree {
    my $this = shift;
    my $eg = $this->event_group();
    return ExSite::Tree("evt_id","parent",$eg->getlist());
}

# ...as a hash

sub event_group_hash {
    my $this = shift;
    return $this->event_group->hash("evt_id");
}

#=== Date queries

# display the date

sub date { &show_date(@_); }
sub show_date {
    my ($this,$type,%opt) = @_;
    my $date = $this->get_date($type);
    if ($date) {
	return $date->show(%opt);
    }
    return undef;
}

# display start and end date as a single value

sub show_date_range {
    my ($this,%opt) = @_;
    my $d = $this->get_date();
    my $dates = $this->get_dates();
    if ($dates) {
	my $ndates = $dates->count;
	if ($ndates == 1) {
	    return $dates->next->show();
	}
	elsif ($ndates > 1) {
	    my $d1 = $dates->first;
	    my $d2 = $dates->last;
	    return $d2->show_date_range($d1->get_datetime,undef,$ndates);
	}
    }
    return undef;
}

# get all event dates as an ObjectList

sub get_dates {
    my ($this) = @_;
    if ($this->loaded()) {
	if ($this->{children}{evt_date}) {
	    return $this->{children}{evt_date};
	}
    }
    return undef;
}

# return best individual date for the event

# get a date as an Event::Date object
# use get_date()->getTime() to get an ExSite::Time object

sub get_date {
    my ($this,$type) = @_;
    my $dates = $this->get_dates();
    if ($dates) {
	if ($dates->count > 0) {
	    my $bestguess;
	    $dates->reset(); # reset iterator
	    while (my $date = $dates->next()) {
		my $dtype = $date->getdata("type");
		if ($type && $type eq $dtype) {
		    $dates->reset();
		    return $date;
		}
		elsif ($dtype eq "start") {
		    $bestguess = $date; 
		}
		elsif (! $bestguess) { 
		    $bestguess = $date;
		}
	    }
	    return $bestguess;
	}
    }
    return undef;
}

# get all event registration dates as a hash keyed on the date type

sub get_registration_dates {
    my ($this) = @_;
    my $fees = $this->get_fees();
    $fees->sort("open","close");
    my @date;
    while (my $fee = $fees->next()) {
	push @date, [ $fee->name, $fee->getdata("open"), $fee->getdata("close") ];
    }
    return @date;
}

# get all event dates as a hash keyed on the date type
### DEPRECATED

sub get_dates_by_type {
    my ($this) = @_;
    if (! $this->{date}) {
	my $dates = $this->get_dates();
	if ($dates) {
	    $dates->reset();
	    while (my $d = $dates->next()) {
		my $dtype = $d->getdata("type");
		$this->{date}{$dtype} = $d;
	    }
	}
    }
    return wantarray ? %{$this->{date}} : $this->{date};
}

# try to fill in any missing dates in the registration schedule
# DEPRECATED?

sub set_registration_dates {
    my ($this) = @_;
    my %regdate = $this->get_registration_dates();
    my $eid = $this->id();
    if (! $regdate{'open registration'}) {
	# missing open registration date
	$regdate{'open registration'} = 
	    $regdate{'close early registration'} ||
	    $regdate{'open early registration'};
	if (! $regdate{'open registration'}) {
	    # still nothing
	    $this->warn("Missing open registration date");
	    if (scalar keys %regdate > 0) {
		# some form of registration is defined
		my $t = new ExSite::Time($this->getdata("ctime"),"sql_timestamp");
		$regdate{'open registration'} = 
		    new Modules::Event::Date(data=>{
			evt_id=>$eid,
			type=>"open registration",
			date=>$t->write("sql_date"),
		    });
	    }
	    else {
		return {};
	    }
	}
    }
    if (! $regdate{'close registration'}) {
	# missing close registration date
	$regdate{'close registration'} = 
	    $regdate{'open late registration'} ||
	    $regdate{'close late registration'} ||
	    $this->{date}{start};
	if (! $regdate{'close registration'}) {
	    # still nothing - this is an irregular event without a 
	    # well-defined start.  However, we do have some sort of 
	    # registration timetable, so we will set the close of 
	    # registration to be tomorrow, so that we will accept
	    # registrations now;
	    $this->warn("Missing close registration date");
	    my $t = new ExSite::Time();
	    $t->add(1,"day");
	    $regdate{'close registration'} = 
		new Modules::Event::Date(data=>{
		    evt_id=>$eid,
		    type=>"close registration",
		    date=>$t->write("sql_date"),
		});
	}
    }
    if (! $regdate{'open early registration'}) {
	# missing open early registration date
	if ($regdate{'close early registration'}) {
	    # however an early registration timetable is partly defined
	    $this->warn("Missing open early registration date");
	    my $t = new ExSite::Time($this->getdata("ctime"),"sql_timestamp");
	    if ($t < $regdate{'close early registration'}->getTime()) {
		# it is before the close of early registration
		$regdate{'close early registration'} = 
		    new Modules::Event::Date(data=>{
			evt_id=>$eid,
			type=>"close early registration",
			date=>$t->write("sql_date"),
		    });
	    }
	}
    }
    if (! $regdate{'close early registration'}) {
	# missing close early registration date
	if ($regdate{'open early registration'}) {
	    # however an early registration timetable is partly defined
	    $this->warn("Missing close early registration date");
	    $regdate{'close early registration'} = 
		$regdate{'open registration'};
	}
    }
    if (! $regdate{'open late registration'}) {
	# missing open late registration date
	if ($regdate{'close late registration'}) {
	    # however a late registration timetable is partly defined
	    $this->warn("Missing open late registration date");
	    $regdate{'open late registration'} = 
		$regdate{'close registration'};
	}
    }
    if (! $regdate{'close late registration'}) {
	# missing close late registration date
	if ($regdate{'open late registration'}) {
	    # however a late registration timetable is partly defined
	    $this->warn("Missing close late registration date");
	    $regdate{'close late registration'} = $this->{date}{start};
	    if (! $regdate{'close late registration'}) {
		# yikes, we have a late registration timetable, but no 
		# actual start date for the event.  Ugh.  Set the late
		# registration close to tomorrow, so that late registration
		# can be performed at the current time.
		my $tomorrow = new ExSite::Time();
		$tomorrow->add(1,"day");
		$regdate{'close late registration'} = 
		    new Modules::Event::Date(data=>{
			evt_id=>$eid,
			type=>"close early registration",
			date=>$tomorrow->write("sql_date"),
		    });
	    }
	}
    }
    return wantarray ? %regdate : \%regdate;
}

# registration period = 
# early, regular, late, or undef (registrations not accepted at the given time)
# DEPRECATED?

sub registration_period {
    my ($this,$time) = @_;
    if (! $this->{registration_period}) {
	$this->{registration_period};
    }
    $time or $time = new ExSite::Time;
    my $timekey = $time->write("sql_date");
    if (! $this->{registration_period}{$timekey}) {
	my %regdate = $this->set_registration_dates();
	if ($regdate{'open registration'}) {
	    if ($time >= $regdate{'open registration'}->getTime() &&
		$time <= $regdate{'close registration'}->getTime()) {
		return $this->{registration_period}{$timekey} = "regular";
	    }
	}
	if ($regdate{'open early registration'}) {
	    if ($time >= $regdate{'open early registration'}->getTime() &&
		$time <= $regdate{'close early registration'}->getTime()) {
		return $this->{registration_period}{$timekey} = "early";
	    }
	}
	if ($regdate{'open late registration'}) {
	    if ($time >= $regdate{'open late registration'}->getTime() &&
		$time <= $regdate{'close late registration'}->getTime()) {
		return $this->{registration_period}{$timekey} = "late";
	    }
	}
    }
    return $this->{registration_period}{$timekey};
}

# get_fees - returns all registration fees that are defined

sub get_fees {
    my $this = shift;
    if ($this->loaded()) {
	if ($this->{children}{evt_fee}) {
	    return $this->{children}{evt_fee};
	}
    }
    return undef;
}

# get_allowed_fees - returns all fees that are available to the current user

sub get_allowed_fees {
    my ($this,$time) = @_;
    my $fees = $this->get_fees();
    my $current_fees = new ExSite::ObjectList(type=>"evt_fee");
    if ($fees) {
	$fees->reset;
	while (my $fee = $fees->next) {
	    if ($fee->allow) {
		$current_fees->push($fee->get());
	    }
	}
    }
    $current_fees->sort("sortkey","name");
    return $current_fees;
}

# get_current_fees - returns all fees that are valid for a certain date

sub get_current_fees {
    my ($this,$time) = @_;
    my $fees = $this->get_fees();
    my $current_fees = new ExSite::ObjectList(type=>"evt_fee");
    if ($fees) {
	$fees->reset;
	while (my $fee = $fees->next) {
	    if ($fee->is_open($time)) {
		$current_fees->push($fee->get());
	    }
	}
    }
    $current_fees->sort("sortkey","name");
    return $current_fees;
}

sub get_current_fees_old {
    my ($this,$time) = @_;
    my $fees = $this->get_fees();
    my $current_fees = new ExSite::ObjectList(type=>"evt_fee");
    if ($fees) {
	my $regperiod = $this->registration_period($time);
	if ($regperiod) {
	    while (my $fee = $fees->next) {
		if ($regperiod eq "early" && $fee->getdata("cost_early")) {
		    $current_fees->push($fee->get());
		}
		elsif ($regperiod eq "regular" && $fee->getdata("cost")) {
		    $current_fees->push($fee->get());
		}
		elsif ($regperiod eq "late" && $fee->getdata("cost_late")) {
		    $current_fees->push($fee->get());
		}
	    }
	}
    }
    $current_fees->sort("sortkey","name");
    return $current_fees;
}

# show_fees - show the fees for this event
# pass an "url" object in the options to make these links
# pass a "registrant" object in the options to provide a registration
#      context that controls which/how fees are displayed

sub show_fees {
    my ($this,%opt) = @_;
    my $ml = &get_obj("ML");
    my $out;

    my $fees = $share{Page} ? 
	$this->get_current_fees() :
	$this->get_allowed_fees();   # allow admin override on ctrl panel
    $fees->reset;
    while (my $fee = $fees->next()) {
	$out .= $fee->show(%opt);
    }

    if (! $out) {
	# no fees
	my $dates = $this->show_registration_dates();
	$out .= $dates ?
	    $ml->p($msg{"Sorry, registration is not open at this time."}).$dates :
	    $ml->p($msg{"Sorry, registrations are not being accepted at this time."});
    }
    return $out ? $ml->div($out,{class=>"EventFees"}) : undef;
}

# show_session_fees - show all fees available to all registrants in this 
# session. This includes the fees for other events in the same group. You 
# must pass in a registrant group hash (see Modules::EvtReg::registrant_group)
# to provide the context. This is a structure with the following contents:
# {
#   by_fee => fee_id => [ array of registration records ]
#   by_name => registrant_name => [ array of registration records ]
# }

sub show_session_fees {
    my ($this,%opt) = @_;
    my $ml = &get_obj("ML");
    my $reg;
    my $out;
    my ($fees,$evts);
    
    my $reggrp = $opt{context} || {};
    
    my $main_registrant_name;
    if ($session{registrant}) {
	my $main_registrant = new Modules::Registration::Registration(data=>$session{registrant});
	$main_registrant_name = $main_registrant->name();
    }

    # display all registrants/fees
    if ($reggrp) {
	foreach my $name (sort keys %{$reggrp->{by_name}}) {
	    $out .= $ml->h3($name);
	    my %fee = &keywise("evt_fee_id",$reggrp->{by_name}{$name});
	    my $evts = $reggrp->{event_group};
	    next if (! $evts);
	    my $feeout;
	    while (my $evt = $evts->next()) {
		my $fees = $share{Page} ? 
		    $evt->get_current_fees() :
		    $evt->get_allowed_fees();   # allow admin override on ctrl panel
		if ($fees->count > 0) {
		    my $parent_registration;
		    my $pevt = $evt->parent_event();
		    if ($pevt && $pevt->has_registration) {
			# parent registration is this registrant's registration in the parent event
			my $pfees = $pevt->get_fees();
			my @preg;
			while (my $fee = $pfees->next()) {
			    if ($fee{$fee->id}) {
				push @preg, $fee{$fee->id};
			    }
			}
			my @preg_sorted = sort {$a->{evt_reg_id} <=> $b->{evt_reg_id}} @preg;
			$parent_registration = $preg_sorted[-1];
		    }
		    else {
			# no parent
			$parent_registration = $session{registrant};
		    }
		    while (my $fee = $fees->next()) {
			$opt{parent_registration} = $parent_registration;
			$opt{registration} = $fee{$fee->id};
			$opt{name} = $name;
			$feeout .= $fee->show(%opt);
		    }
		}
	    }
	    if ($feeout) {
		$out .= $ml->div($feeout,{class=>"EvtSessionRegistrant"});
	    }
	}
    }
    return $out ? $ml->div($out,{class=>"EventFees"}) : undef;
}

# show_new_session_fees - show the fees to create a new registrant in this
# group, ie. main event fees

sub show_new_session_fees {
    my ($this,%opt) = @_;
    my $out;
    my $main_evt = $this->main_event();
    if ($main_evt) {
	$out .= $main_evt->show_fees();
    }
    return $out ? $ml->div($out,{class=>"EvtSessionRegistrant"}) : undef;
}

sub show_registration_dates {
    my ($this,%opt) = @_;
    my $ml = &get_obj("ML");
    my $out;
    my $fees = $this->get_fees();
    $fees->sort("open","close");
    my $r = new ExSite::ReportBuilder(title=>"Registration Schedule");
    $r->headers("Registration Type","Registration Opens","Registration Closes");
    $r->nodata("No registration dates are defined.");
    while (my $fee = $fees->next()) {
	$r->push($fee->name,$fee->show_open_date,$fee->show_close_date);
    }
    $r->set("class","Report EvtRegSchedule");
    return $r->make();
}

# registrations - return array of registrations to this event that have
# been booked by the current user

sub registrations {
    my $this = shift;
    my $reg;
    if (require Modules::Finance::Account) {
	$reg = new ExSite::ObjectList;
	my $acct = new Modules::Finance::Account;
	$acct->setup_my_account();
	if ($acct->defined) {
	    $this->setup_querylib();
	    $reg->setlist( $share{DB}->get_query("my event registrations",$acct->id) );
	}
    }
    return $reg;
}

#=== DB

# load the record and metadata

sub loaded {
    my $this = shift;
    if (! $this->{data} || ! $this->{children}) {
	return $this->load();
    }
    return $this->{data};
}

sub load {
    my $this = shift;
    if ($this->SUPER::load()) {
	$this->load_children();
    }
    return $this->{data};
}

sub load_children {
    my ($this,@child) = @_;
    if (@child > 0) {
	foreach my $c (@child) { 
	    if (! exists $this->{children}{$c}) {
		$this->{children}{$c} = $this->get_children($c);
	    }
	}
    }
    else {
	$this->{children} = $this->get_children();
    }
    return wantarray ? %{$this->{children}} : $this->{children};
}

sub copy_event {
    my ($this,$template_id,$name) = @_;
    my $evt = new Modules::Event::Event(id=>$template_id);
    my $evt2 = new Modules::Event::Event(id=>$evt->copy());
    my $stat = 0;
    if ($evt2->defined) {
	$stat++;
	my %children = $evt->get_children();
	while (my $edate = $children{evt_date}->next()) {
	    $edate->setdata("evt_id",$evt->id);
	    $stat++ if $edate->copy();
	}
	while (my $eloc = $children{evt_location}->next()) {
	    $eloc->setdata("evt_id",$evt->id);
	    $stat++ if $eloc->copy();
	}
	while (my $efee = $children{evt_fee}->next()) {
	    $efee->setdata("evt_id",$evt->id);
	    $stat++ if $efee->copy();
	}
    }
    return $stat;
}

sub setup_querylib {
    my $this = shift;
    if (! $this->{querylib_setup}) {
	my $db = $share{DB};
	$db->set_query("activities",
		       ( 
			 sql => "select e.*,ed.date startdate,ed.time starttime from evt e,evt_date ed where ed.evt_id=e.evt_id and e.parent=? order by ed.date,ed.time",
			 nparam => 1,
			 mode => "r",
			 keys => ["evt","evt_date"],
			 )
		       );
	$db->set_query("my event registrations",
		       ( sql=>"select r.* from evt_reg r,evt_fee f,evt e where f.evt_id=e.evt_id and r.evt_fee_id=f.evt_fee_id and r.account_id=? order by r.evt_reg_id",
			 nparam=>1,
			 mode=>"r",
			 keys=>["evt","evt_reg","evt_fee"],
			 ));
    }
    $this->{querylib_setup} = 1;
}

1;

