package Modules::MemberDirectory;

#----------------------------------------------------------------------------
#
#   Copyright (C) 2010 - Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
#  Organize members into browsable categories
#
#----------------------------------------------------------------------------

use strict;

# useful kernel libs; you can/should remove any of these that are not needed

use ExSite::Config;          # required
use ExSite::Input;           # optional input manager
use ExSite::Misc;            # optional utils
use ExSite::Util;            # optional utils
use ExSite::ML;              # optional markup-language generation
use ExSite::FormBuilder;     # optional form generation
use ExSite::ReportBuilder;   # optional report generation
use ExSite::Form;            # optional database manager
use ExSite::Tree;
use ExSite::HTML;
use ExSite::Object;
use ExSite::Module;

# recommended base class for plug-in modules

use Modules::BaseDCD;
use Modules::Membership;

# declare package globals

use vars qw(@ISA $ml $depth);       # Perl inheritance global & markup language object

# define our class inheritance

@ISA = qw(Modules::BaseDCD); # inherit from this base class

sub read {
    my ($this,$options) = @_;
    my $db = $share{DB};
    
    $db->set_query("my membership categories",
		   sql=>"select cat.* from member_category cat, member_category_join jcat where cat.member_category_id=jcat.member_category_id and jcat.member_id=?",
		   nparam=>1,
		   mode=>"r",
		   keys=>["member_category", "member_category_join"]);

	my $per_page = &preference("Membership.records_per_page") || 200;
	my $offset = ($this->page()-1) * $per_page;

	# add attribute fields
	my $membership = &ExSite::Module::get_module("Membership");
	my @meta = $membership->get_all_meta;
	my (@extracol,@extratab,@extracond,$join);
	my $nattr = 1;
	my $join = "member m left join member_category_join jcat on m.member_id = jcat.member_id left join member_attribute a0 on a0.member_id=m.member_id";
	foreach my $meta (@meta) {
	push @extracol,"a$nattr.value $meta";
	if ($join) {
		$join = "($join) left join member_attribute a$nattr on a$nattr.member_id=m.member_id and a$nattr.name='$meta'";
	}
	$nattr++;
	}
	my $select = "m.*,jcat.member_category_join_id" . "," . join(",",@extracol);
	my $sort = &preference("Membership.sort_field") || "last_name";
	my $where = "jcat.member_category_id=? and m.visibility = 'visible'";
	$where .= ' and m.type in ' . $membership->fetch_member_type_sql() if ($membership->fetch_member_type_sql);

	my $sql = "select $select from $join where $where group by m.member_id order by ? limit ?,$per_page";

    $db->set_query("members in category",
		   sql=>$sql,
		   nparam=>3,
		   mode=>"r",
		   keys=>["member", "member_category_join"]);

	my $sql = "select $select from $join where $where group by m.member_id order by ?";

    $db->set_query("all members in category",
		   sql=>$sql,
		   nparam=>2,
		   mode=>"r",
		   keys=>["member", "member_category_join"]);
		   
	my $privacy_sql;
	if (!$db->is_manager && &preference("MemberDirectory.use_privacy")) {
	my $privacy = "('public')";
	$privacy = "('public','members')" if ($db->authorize() == 1);
	$privacy_sql = "and m.privacy in $privacy"
	}    
	my $sql = "select member_category_id,count(*) as count from member_category_join mcj,member m where m.member_id = mcj.member_id and m.visibility = 'visible' $privacy_sql group by member_category_id";
	$db->set_query("member counts by category",
		sql=>$sql,
		nparam=>0,
		mode=>"r",
		keys=>["member_category_join"]);
    return $this->SUPER::read($options);
}

sub path {
    my $this = shift;
    my @path;
    my $cat = $this->category;
	if (!$cat) {
	return wantarray ? () : [];
	}
    push @path, $cat;
    while ($cat->getdata("parent_id")) {
	$cat = $this->category($cat->getdata("parent_id"));
	unshift @path, $cat;
    }
    return wantarray ? @path : \@path;
}

# catid = member_category_id
# mid = member_id
sub is_member_in_category {
	my ($this,%opt) = @_;
	my $catid = $opt{catid};
	my $mid = $opt{mid} || $this->DB->my_uid();
	my $cat = $share{DB}->get_query("my membership categories",$mid);	
	$this->{member_categories}{$mid} = &keywise("member_category_id",$cat);	
	($this->{member_categories}{$mid}{$catid}) ? return 1 : return 0;
}

sub leaf_index {
	my ($this) = @_;
	return $this->{tree_leaves} if ($this->{tree_leaves});
	# id = id of starting node
    my $list = $this->tree()->get_leaves();
    return $this->{tree_leaves} = &keywise("member_category_id",$list);
}

# display tree as a collapsible checkbox tree
# the starting node is NOT included
sub ctree {
	my ($this,$id,@keys) = @_;
	my ($out,@node);
	if (exists ($this->tree()->{node}{$id})) {
	@node = @{$this->tree()->{node}{$id}{child}};
	}
	else {
	@node = $this->tree()->get_topnodes();
	}
	$depth = 0;
	foreach my $n (@node) {
	if (@keys == 0) { @keys = ($n->{id_key}); }
	foreach my $k (@keys) {
		my $id = $n->{data}{$n->{id_key}};
		my $checked = $this->is_member_in_category(catid=>$id) ? 1 : 0;
		my $input_name = "cat";
		$input_name  = "cat_p" if (!$this->leaf_index()->{$id} && $this->is_leaf_only());
		my $options = { type=>"checkbox", name=>$input_name, value=>$id };
		$options->{checked} = "checked" if ($checked);
		my $input;
		$input = $ml->input(undef, $options) 
			if (!$this->is_leaf_only() || $this->leaf_index()->{$id});
		$out .= "\n<li>".$input."\n".$n->{data}{$k};
	}
	$depth++;
	$out .= $ml->ul($this->ctree_r($n,@keys))."</li>\n";
	$depth--;
	}
	return $out;
}

sub ctree_r {
	my ($this,$node,@keys) = @_;
	my ($out,@node);
	foreach my $n ($this->tree()->get_child_nodes($node->{data}{$node->{id_key}})) {
	if (@keys == 0) { @keys = ($n->{id_key}); }
	foreach my $k (@keys) {
		my $id = $n->{data}{$n->{id_key}};
		my $checked = $this->is_member_in_category(catid=>$id) ? 1 : 0;
		my $input_name = "cat";
		$input_name  = "cat_p" if (!$this->leaf_index()->{$id} && $this->is_leaf_only());
		my $options = { type=>"checkbox", name=>$input_name, value=>$id };
		$options->{checked} = "checked" if ($checked);
		my $input;
		$input =  $ml->input(undef, $options) 
			if (!$this->is_leaf_only() || $this->leaf_index()->{$id});
		$out .= "\n<li>".$input."\n".$n->{data}{$k};
	}
	$depth++;
	my $subtree = $this->ctree_r($n,@keys);
	$out .= "\n".$ml->ul($subtree) if ($subtree);
	$out .= "</li>";
	$depth--;
	}
	return $out;
}

sub js {
	my ($this) = @_;
	my $basepath = "$config{server}{server}$config{server}{HTMLpath}/_Modules/MemberDirectory";
	return <<END;
<script type="text/javascript" src="$basepath/js/jquery.collapsibleCheckboxTree.js"></script>
<link rel="stylesheet" href="$basepath/css/jquery.collapsibleCheckboxTree.css" type="text/css" />
<script type="text/javascript">
jQuery(document).ready(function(){
\$('ul#ctree').collapsibleCheckboxTree({
	checkParents: false
});
});
/*
jQuery(document).ready(function(){
		\$('ul#example').collapsibleCheckboxTree({
		checkParents : true, // When checking a box, all parents are checked (Default: true)
		checkChildren : false, // When checking a box, all children are checked (Default: false)
		uncheckChildren : true, // When unchecking a box, all children are unchecked (Default: true)
		initialState : 'default' // Options - 'expand' (fully expanded), 'collapse' (fully collapsed) or default
												});
});
*/
</script>
END
}

sub member_update {
	my ($this) = @_;
	my $out;
   	$out .= $this->js();
   	$out .= $ml->h1("Update your Categories");
	if (!$this->DB->authorize) {
	$out .= $ml->p($this->error($msg{"You are not logged in."}));
	return $out;
   	}
   	my $type = $this->DB->my_user_record()->{type};
	if (scalar grep(/^$type$/,@{$config{MemberDirectory}{types_allowed}}) <= 0) {
		my $types = join(", ",@{$config{MemberDirectory}{types_allowed}});
		return $ml->p($this->error(&substitute($msg{"You are a [[type]] member and may not configure the member directory."},
			{type=>$type})));
	}
   	my $max_checked = &preference("MemberDirectory.max_categories") || 6;
	my $fb = new ExSite::FormBuilder;
   	$out .= $ml->p($msg{&substitute("Choose up to [[max_checked]] categories from the list below.\n
   	Your selections will be used to populate your profile in the directory.\n
   	Click on the '+' next to a category to see its subcategories.",
   		{max_checked=>$max_checked})});
	$out .= $ml->p($ml->strong("Note: You must click submit to save your changes."));
	my $ui_tree .= $fb->buttons(submit=>1,reset=>1);
	$ui_tree .= $ml->br();
	$ui_tree .= "\n".$ml->ul($this->ctree(undef,"category"),{id=>"ctree"});
	$ui_tree .= $fb->buttons(submit=>1,reset=>1);
	$out .= $ml->form($ui_tree,{action=>
		$this->link(__plaintext=>1,md=>"do_member_update"),method=>"post"});	
	$out .= <<END;
<script>
\$.fn.limit = function(n) {
 var self = this;
 this.click(function(){
   (self.filter(":checked").length==n)?
     self.not(":checked").attr("disabled",true).addClass("disabled"):
     self.not(":checked").attr("disabled",false).removeClass("disabled");
 });

 this.mousedown(function(){
   (self.filter(":checked").length>n)?
     self.not(":checked").attr("disabled",true).addClass("disabled") : 0;
 });

 jQuery(document).ready(function(){
   (self.filter(":checked").length==n)?
   self.not(":checked").attr("disabled",true).addClass("disabled"):
   self.not(":checked").attr("disabled",false).removeClass("disabled");
 });
}
\$("input[name=cat]").limit($max_checked);
</script>
END
	return $out;	
}

sub do_member_update {
	my ($this) = @_;
	my $out;
	my $mid = $this->DB->my_uid;
	if (!$mid) {
	$out .= $ml->p($this->error($msg{"You are not logged in."}));
	return $out;
   	}	
	$this->DB->delete("member_category_join",{member_id=>$mid});
	my @catid = split(/; /,$this->{input}{cat});
	if (!scalar @catid) {
		my $msg;
		if ($this->is_leaf_only()) {
			$msg = $msg{"You must choose at least 1 category which does not have any further sub-categories."};
			return $this->error($msg);
		}
		$msg = $msg{"You must choose at least 1 category."};
		return $this->error($msg);
	}
	foreach my $catid (@catid) {
		$this->DB->insert("member_category_join", {
			member_id=>$mid,
			member_category_id=>$catid
		});
	}
	$out .= $ml->div($msg{"Your categories have been updated."},{class=>"info"});
	$out .= $this->member_update();
	return $out;
}

# write method (builds output for regular web pages)

sub write {
    my ($this,$options) = @_;

    my %opt = &DecodeString($options);
    my $uid = $opt{uid} || $this->DB->my_uid;
    my $member = $share{DB}->fetch("member",$uid);
    $ml = new ExSite::ML;
    my $out;
	my $action = $this->{input}{md} || $opt{action};
	my $catid  = $this->{input}{catid} || $opt{catid};
	$this->{category} = $this->DB->fetch("member_category",$catid);
    if ($action eq "my_categories" && $uid) {
    	my $type = $member->{type};
	    if (scalar grep(/^$type$/,@{$config{MemberDirectory}{types_allowed}}) > 0) {
		my @cat = $share{DB}->get_query("my membership categories",$uid);
		my $list;
		foreach my $cat (@cat) {
			my $catid = $cat->{member_category_id};
			my $tree = $this->tree();
			my @parents;
			my $current_id = $catid;
			while (my $parent = $tree->get_parent_node($current_id)) {
				push(@parents,$parent->{data}{category});
				$current_id = $tree->id($parent);
			}
			my $parents = join(" > ", reverse(@parents));
			$list .= $ml->li($parents.$ml->br().$ml->a($cat->{category},
				{href=>$this->link(catid=>$catid,page=>undef)}
			));
		}
		my $button;
		if ($uid == $this->DB->my_uid()) {
			$button = &ExSite::HTML::Button(label=>$msg{update},
				url=>$this->link(md=>"member_update"),type=>"button");
		}
		$out .= $ml->p($ml->strong($msg{"Listed under"})."&nbsp;".$button) . $ml->ul($list);
		}
	} elsif ($action eq "do_member_update") {
		return $this->do_member_update();
    } elsif ($action eq "member_update") {
		return $this->member_update();
    }
	else {
		$out .= $this->show(%opt);
	}
	return undef if (!$out);
    return $ml->div($out,{class=>"MemberDirectory"});
}

sub show {
	my ($this,%opt) = @_;
	my $tree = $this->tree();
	my $out;	
    # find starting root node
    my $catid;
    my @topnodes = $tree->get_topnodes;
	my @c;
	if ($this->category) {
		$catid = $this->category->id();
	    @c = $tree->get_child_data($catid);	    		
		$this->set_page_title;
	} elsif ($opt{category}) {
	foreach my $n (@topnodes) {
		$catid = $n->{data}{member_category_id} if ($n->{data}{category} eq $opt{category});
	    @c = $tree->get_child_data($catid);		
		$this->set_page_title;		    
	}
	} else {
		@c = $tree->get_topnodes_data();
	}
	my ($titles,$panes,$states,$links);
    foreach my $data (@c) {
		my $count = $this->count_members($data->{member_category_id});
		push(@$titles,$data->{category} . "&nbsp;($count)");
		my @c = $tree->get_child_data($data->{member_category_id});
    	my $pane = $this->sublist(@c);
    	push(@$panes,$pane);
		push(@$states,0);
		push(@$links,$this->link(catid=>$data->{member_category_id},page=>undef));
    }
	my @path = $this->path();
	my @menu;
    push @menu, $ml->a("Top",{ href=>$this->link(pro=>"member_directory_module",
    	catid=>undef,
    	page=>undef,
		_id=>&ExSite::Module::service_page("Membership")->id())});
	foreach my $cat (@path) {
		push(@menu,$ml->a($cat->getdata("category"), {href=>$this->link(catid=>$cat->id,page=>undef)}));
	}
	$out .= $ml->div(join("&nbsp;&gt;&nbsp;",@menu),{id=>"MemberCategoryTopMenu"});
	if (scalar @c) {
	$out .= $ml->h3($this->category_name . " Sub-categories");
	$out .= &ExSite::HTML::AccordionBox(
		titles=>$titles,
		panes=>$panes,
		states=>$states,
		links=>$links);
	}
	if ($this->category) {
	if ($this->leaf_index()->{$this->category->id}) {
	my $per_page = &preference("Membership.records_per_page") || 200;
	my $offset = ($this->page()-1) * $per_page;
	my $count = $this->count_members($this->category->id);
	my @members = $this->DB->get_query("members in category",($catid,"organization",$offset));
	my $membership = &ExSite::Module::get_module("Membership");
	$out .= $ml->h3($msg{$this->category->getdata("category")} . " " . "Listing");
	$out .= $membership->index(\@members,$count);
	}
	}
	return $out;
}

sub category {
	my ($this,$catid) = @_;
	my $catid = $catid || $this->{input}{catid};
	return undef if (!$catid);
	my $cat = new ExSite::Object(type=>"member_category",id=>$catid);
	return $cat;
}

sub category_name {
	my ($this) = @_;
	return undef if (!$this->category);
	return $this->category->getdata("category");
}

sub set_page_title {
	my ($this) = @_;
	my $title = $this->category_name . " | " . $config{site}{name};
	$share{Page}->set_metadata("title",$title);
}

sub sublist {
	my ($this,@nodes) = @_;
	return undef if (!scalar @nodes);
	my @list;
	foreach my $data (@nodes) {
		my $catid = $data->{member_category_id};
		my $count = $this->count_members($catid);
		my $link = $ml->a($data->{category},
			{href=>$this->link(pro=>"dir",catid=>$catid,page=>undef)});
		push(@list,$link . "&nbsp;($count)");
	}
	return $ml->p(join(', ',@list));
}

sub count_members {
	my ($this,$catid) = @_;
	if (!$this->{member_category_count}) {
	my $sql = "select member_category_id,count(*) as count from member_category_join group by member_category_id";
	my $counts = $this->DB->get_query("member counts by category");	
	$this->{member_category_count} = &keywise("member_category_id",$counts);
	}
	my @subnodes = $this->tree()->collapse($catid);
	my @ids = map { $_->{member_category_id} } @subnodes;
	unshift(@ids,$catid);
	my $count;
	foreach my $id (@ids) {
		$count += $this->{member_category_count}{$id}{count};
	}
	return $count;
}

sub DB {
	return $share{DB};
}

sub page { 
	my ($this) = @_;
	my $page = $this->{input}{page} || 1;
	return $page;
}

# ioctl method (used by ExSite to query the module for its functionality)

sub ioctl {
    my $this = shift;
    $_ = shift;           # $_ is the ioctl request

    if (/isRestricted/) {
	return 0;
    }
    elsif (/isService/) {
	return 1;
    }
    elsif (/ModuleName/) {
	return "Membership Directory";
    }
    elsif (/ModuleInfo/) {
	return "Organize members into browsable categories";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/Search/) {
	return \&search_index;
    }
    elsif (/Dependencies/) {
	return "Membership";
    }    
}

#----------------------------------------------------------------------------
# Everything after this point consists of private methods.

# ctrl_panel() generates the contents of the administrator control panel

sub ctrl_panel {
    my $this = shift;
    $ml = new ExSite::ML;
    my $section_id = $this->get_section_id;
    my $out;
    my $db = $share{DB};
    my $tree = $this->tree();

    $out .= $this->set_section_id;
    my @topcat = $tree->get_topnodes_data();
    my @category_item;
    foreach my $cat (@topcat) {
	next if ($cat->{section_id} && $cat->{section_id} != $section_id);
	push @category_item, $this->add_category_item($cat,$tree);
    }
    $out .= &ExSite::HTML::DynList(@category_item);

    if ($this->{input}{mcat}) {
	$out .= $this->show_category($this->{input}{mcat},$tree);
    }

    return $out;
}

sub tree {
	my ($this) = @_;
	return $this->{tree} if ($this->{tree});
	my @cat = $share{DB}->fetch_all("member_category");
	return $this->{tree} = new ExSite::Tree("member_category_id","parent_id",@cat);
}

sub add_category_item {
    my ($this,$category,$tree) = @_;
    my $id = $category->{member_category_id};
    my %item;
    $item{label} = $category->{category};
    $item{url} = $this->link(mcat=>$id);
    #$item{text} = "members edit delete";
    my @cat = $tree->get_child_data($id);
    my @list;
    if (@cat > 0) {
	foreach my $cat (@cat) {
	    push @list, $this->add_category_item($cat,$tree);
	}
	$item{sublist} = \@list;
    }
    return \%item;
}

sub show_category {
    my ($this,$id,$tree) = @_;
    my $out;
    my $category = new ExSite::Object(type=>"member_category",id=>$id);
    my $out = $ml->h1($category->name);

    my @options;
    push @options, ["", "== select membership category =="];
    my @topcat = $tree->get_topnodes_data();
    foreach my $cat (@topcat) {
	&add_member_category($this,0,$tree,$cat,\@options);
    }
    my $fb = new ExSite::FormBuilder(method=>"post",action=>$this->link());
    $fb->input(prompt=>"Category",name=>"category",value=>$category->getdata("category"),type=>"text",size=>60);
    $fb->input(prompt=>"Parent",name=>"parent_id",options=>\@options,value=>$category->getdata("parent_id"),type=>"select");
    $out .= $fb->make();

    if ($this->{input}{dircmd} eq "del" && $this->{input}{cj}) {
	$share{DB}->trash_key("member_category_join",$this->{input}{cj});
    }

    my @member = $this->DB->get_query("all members in category",($id,"organization"));
    $out .= $ml->h3("Members in this category");
    if (@member > 0) {
	$out .= $ml->p("Click the member name to remove them from this category");
	foreach my $data (@member) {
	    my $member = new Modules::Membership::Member(data=>$data);
	    $out .= 
		$ml->a($member->name,{href=>$this->link(cj=>$data->{member_category_join_id},dircmd=>"del")}).
		$ml->br;
	}
    }
    else {
	$out .= $ml->p($msg{"No members found in this category."});
    }

    return $out;
}

sub add_member_category {
    my ($this,$level,$tree,$cat,$options) = @_;
    my $label = ". . " x $level;
    $label .= $cat->{category};
    push @$options, [$cat->{"member_category_id"}, $label];
    my @subcat = $tree->get_child_data($cat->{member_category_id});
    $level++;
    foreach my $cat (@subcat) {
	&add_member_category($this,$level,$tree,$cat,$options);
    }
}

# search_index() allows this plug-in to manipulate the system search index.

sub search_index {
    my ($this,$search,$site,$page) = @_;

    # $search is an ExSite::Search object.  Use this to modify the
    # search index.

    # $site is an ExSite::Section object for the current section.  It is
    # normally used to identify information that should be indexed in
    # this call.

    # $page is an ExSite::Page object for the service page that 
    # all search hits will be directed to.

    # You can optionally return some status messages
    my $out;

    # manipulate the search index here.

    return $out;
}

sub is_leaf_only {
	my ($this) = @_;
	return 1;
	return $this->config->{leaf_only};
}

sub config {
	my ($this) = @_;
	return $config{$this->module_name};
}

sub member {
	my ($this) = @_;
	if (!$this->{member}) { 
		$this->{member} = new Modules::Membership::Member(id=>$this->DB->my_uid);
	}
	return $this->{member};
}

1;
