package Modules::Poll;

#----------------------------------------------------------------------------
#
#   Copyright (C) 2010 - Matt Wong
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;

# useful kernel libs; you can/should remove any of these that are not needed

use ExSite::Config;          # required
use ExSite::Input;           # optional input manager
use ExSite::Misc;            # optional utils
use ExSite::Util;            # optional utils
use ExSite::ML;              # optional markup-language generation
use ExSite::FormBuilder;     # optional form generation
use ExSite::ReportBuilder;   # optional report generation
use ExSite::Form;            # optional database manager

# recommended base class for plug-in modules

use Modules::BaseDCD;

# declare package globals

use vars qw(@ISA $ml);       # Perl inheritance global & markup language object

# define our class inheritance

@ISA = qw(Modules::BaseDCD); # inherit from this base class



# write method (builds output for regular web pages)

sub write {
	my ($this,$options) = @_;
	$ml = new ExSite::ML;
	
	my %opt = &DecodeString($options);
	my $cmd = $opt{cmd} || $this->{input}{cmd};

	my $out;

	if ($cmd =~ /^(display_results|display_poll)$/) {
		my $pid = $opt{pid} || $this->{input}{pid};
		$out .= $this->$cmd($pid);
	}
	else {
		$out .= $this->display_polls();
	}

	return $this->wrap($out);
}

sub wrap {
	my ($this,$body) = @_;
	
	my $out .= $ml->link(undef, {
		rel => "stylesheet",
		type => "text/css",
		href => "$config{server}{server}$config{server}{HTMLpath}/_Modules/Poll/Poll.css",
	});
	$out.= $body;
	
	return $ml->div($out, { class => "Poll" } );
}

# ioctl method (used by ExSite to query the module for its functionality)

sub ioctl {
	my $this = shift;
	$_ = shift;           # $_ is the ioctl request

	if (/isRestricted/) {
		return 0;
	}
	elsif (/isService/) {
		return 1;
	}
	elsif (/ModuleName/) {
		return "Polls/Quizzes";
	}
	elsif (/ControlPanel/) {
		return \&ctrl_panel;
	}
	elsif (/DynContentParam/){
		return $this->getDynContentParameter(shift);
	}
}

sub getDynContentParameter {
	my ($this,$section_id) = @_;
	my $polls = $this->{DB}->fetch_match("poll", { section_id => $section_id });
	my $out;
	my @options;
	
	push(@options, ["option", "< Default >", { value => "" }]);
	foreach my $poll (@$polls) {
		push(@options, ["option", $poll->{question}, { value => "pid=" . $poll->{poll_id} . "&cmd=display_poll" }]);
	}

	$out .= $ml->table(
		$ml->tr(
			$ml->td($ml->b("Choose a poll: ")) .
			$ml->td(
				$ml->select(\@options, { name => "param" })
			)
		)
	);
	
	return $out;
}

sub setup {
	my ($this,%opt) = @_;

	my $in = new ExSite::Input;
	$this->{input} = $in->combine;
	
	my $query = $in->query();
	foreach my $param (keys %$query) {
		$this->{input}{$param} = $query->{$param};
	}
	
	$this->{DB} = $share{DB};
	$this->{ML} = $ml = new ExSite::ML();
	$this->{level} = $this->{DB}->authorize;
	$this->{query} = $query;
	
	return;
}

#----------------------------------------------------------------------------
# Everything after this point consists of private methods.

# ctrl_panel() generates the contents of the administrator control panel

sub ctrl_panel {
	my $this = shift;
	$this->{section_id} = $this->{input}{section_id};

	my $section_id = 
	my $out;  # our output buffer
	my $cmd = $this->{input}{cmd};
	
	if ($cmd =~ /^(insert_update_poll|delete_poll|insert_poll|vote_report)$/) {
		$out .= $this->$cmd();
	}
	elsif ($this->{input}{section_id} || $cmd=~ /^(poll_report)$/) {
		$out .= $this->poll_report();
	}
	else {
		$out .= $this->set_section_id();
	}
	
	return $out;
}

sub delete_poll {
	my $this = shift;
	my $out;
	
	if ($this->{input}{pid}) {
		my $poll = $this->{DB}->fetch("poll", $this->{input}{pid});
		$this->{DB}->trash_key("poll" ,$this->{input}{pid});
		
		$out .= $this->{ML}->h1("Deleted Poll: " . $poll->{question});	
	}
	
	$out .= $this->poll_report();
	return $out;
}

sub insert_update_poll {
	my $this = shift;
	
	my $f = $this->{DB};
	my $pid = $this->{input}{pid};
	
	if (!$this->{input}{action}) {
		my $poll = $this->{DB}->fetch("poll", $pid) if ($pid);

		$f->input_column(table => "poll",
				 column => "section_id",
				 value => $poll->{section_id} || $this->{section_id},
				 record => $pid);
		$f->input_column(table => "poll",
				 column => "question",
				 value => $poll->{question},
				 required => 1,
                                 record => $pid);	
		$f->input_column(table => "poll",
				 column => "answers",
				 value => $poll->{answers},
				 required => 1,
                                 record => $pid);				 
		$f->input_column(table => "poll",
				 column => "solution",
				 value => $poll->{solution},
                                 record => $pid);
		$f->input_column(table => "poll",
				 column => "result_type",
				 value => $poll->{result_type} || "all",
                                 record => $pid);				 
		$f->input_column(table => "poll",
				 column => "start_date",
				 value => $poll->{start_date} || "---",
                                 record => $pid);	
		$f->input_column(table => "poll",
				 column => "end_date",
				 value => $poll->{end_date} || "---",
                                 record => $pid);					 
		$f->input_column(table => "poll",
				 column => "privacy",
				 value => $poll->{privacy} || "public",
                                 record => $pid);	
		$f->input_column(table => "poll",
				 column => "message",
				 value => $poll->{message},
                                 record => $pid);
		$f->input_column(table => "poll",
				 column => "secret",
				 value => $poll->{secret} || "Y",
                                 record => $pid);
		
		my $db_action = $pid ? "update" : "insert";
		$f->input_html(
			type => "hidden",
			name => "action",
			value => $db_action,
		);		
		
		if ($pid) {
			$f->input_html(
				type => "hidden",
				name => "poll#$pid#poll_id",
				value => $pid,
			);
		}
		
		$f->form->action($this->link(cmd=>"insert_update_poll"));
	}
	else {
		$f->do($this->{input}{action});
		return $this->poll_report();
	}
	
	my $header = $pid ? "Update Poll" : "Add Poll";
	return $this->{ML}->h1($header) . $f->form->make();	
}

=head
sub reset_link {
	my ($this,%new_parameters) = @_;
	foreach my $key (keys (%{$this->{query}})) {
		# we set any parameters that are in the old query 
		# but not in the new query to undef excluding special cms params
		if ( !exists $new_parameters{$key} && $key !~ /^_/i && $key !~ /sid/) {
			$new_parameters{$key} = undef;
		}
	}
	return $this->link(%new_parameters);
}
=cut

sub poll_report {
	my $this = shift;
	
	my $r = new ExSite::ReportBuilder();		
	$r->init(title => "Polls/Quizzes",
		headers => ["Question", "Type", "Start Date", "End Date", "options"],
		nodata => "No Polls or Quizzes found.",
		tools => $this->new_poll_button(),
	);
	
	my @polls = $this->{DB}->fetch_match("poll", { section_id => $this->{section_id} });
	foreach my $poll (@polls) 
	{
		my @row = $this->report_row($poll);
		$r->push(@row) if (scalar @row);
	}
	
	my $out = $r->make;
	return $out;
}

sub new_poll_button {
	my $this = shift;
	
	my $newAttributes = { 
		    src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{new}{img},
		    border => "0",
		    alt => "new",
		    title => "new"
	};
	
	my $new = $this->link(cmd => "insert_update_poll", pid => undef);
	my $out = $share{ML}->a($share{ML}->img(undef, $newAttributes), { href => $new });
	return $out;
}

sub report_row {
	my ($this,$poll) = @_;
	
	my $edit = $this->link(cmd => "insert_update_poll", pid => $poll->{poll_id});
	my $view = $this->link(cmd => "vote_report", pid => $poll->{poll_id});
	my $delete = $this->link(cmd => "delete_poll", pid => $poll->{poll_id});

	my $viewAttributes = { 
		    src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{report}{img},
		    border => "0",
		    alt => "report",
		    title => "report"
	};
	my $editAttributes = { 
		src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{edit}{img},
		border => "0",
		alt => "edit",
		title => "edit"
	};
	my $deleteAttributes = {
		src => $config{server}{HTMLpath} . "/_Modules/WebDB/" . $config{db_ops}{delete}{img},
		border => "0",
		alt => "delete",
		title => "delete"
	};
		
	my $imageLinks = $share{ML}->a($share{ML}->img(undef, $viewAttributes), { href => $view, style =>"padding-right: 5px;" }) . 
		$share{ML}->a($share{ML}->img(undef, $editAttributes), { href => $edit, style =>"padding-right: 5px;" }) .
		$share{ML}->a($share{ML}->img(undef, $deleteAttributes), { href => $delete }) . "\n";

	my $t = new ExSite::Time($poll->{start_date}, "sql_datetime");
	my $start_date;
	if ($poll->{start_date} eq "0000-00-00") { $start_date = "N/A"; }
	else { $start_date = $t->write("date_long"); }
	
	$t->set($poll->{end_date}, "sql_datetime");
	my $end_date;
	
	if ($poll->{end_date} eq "0000-00-00") { $end_date = "N/A"; }
	else { $end_date = $t->write("date_long"); }
		
	my $type = $poll->{solution} ? "Quiz" : "Poll";

	return (
		$poll->{question},
		$type,
		$start_date,
		$end_date,
		$imageLinks
	);
}

sub display_polls {
	my $this = shift;
	my $out;
	
	my $t = new ExSite::Time;
	my $today = $t->write("sql_date");
	my $site = $this->{DB}->this_site();
	my $section_where;

	if ($site) {
		$section_where = "AND section_id = " . $site->{section_id};
	}
	
	my @polls = $this->{DB}->custom_query("SELECT * FROM poll WHERE ((start_date <= '$today' AND end_date >= '$today') OR (start_date = '0000-00-00' AND end_date = '0000-00-00')) $section_where");
	
	return $msg{"There are no polls at this time."}
		if (!@polls);
	
	if (scalar @polls > 1) {
		$out .= $this->poll_list(\@polls);
	}
	else {
		my $poll = $polls[0];
		$out .= $this->display_poll($poll->{poll_id});
	}
		
	return $out;
}

sub poll_list {
	my $this = shift;
	my $polls = shift;
	my $out;
			
	foreach my $poll (@$polls) {
		my $poll_type;
	
		if ($poll->{solution}) { $poll_type = "Quiz"; }
		else { $poll_type = "Poll"; }
	
		$out .= $ml->a($poll_type . ": " . $poll->{question}, 
			{ href => $this->link(cmd => "display_poll", pid => $poll->{poll_id}) }) . $ml->br();
	}
	
	return $ml->h1("Polls/Quizzes") . $out;
}

sub display_poll {
	my $this = shift;
	my $poll_id = shift;
	my $out;
	
	my $poll = $this->{DB}->fetch("poll", $poll_id);
	
	if (!$this->is_poll_active($poll)) {
		return $this->display_results($poll_id);
	}
	
	my $answer_input;
	my @answers = split(/\|/, $poll->{answers});
	foreach my $answer (@answers) {
		$answer_input .= $ml->input(undef, { type => "radio", name => "poll_answer", value => $answer }) . 
			" " . $msg{$answer} . $ml->br();
	}
	
	my $results_link = " - " . $ml->a($msg{"see results"}, { href => $this->link(pid => $poll_id, cmd => "display_results") }) 
		if ($poll->{secret} eq "N");
	
	my $button_label;
	my $header;
	if ($poll->{solution}) { 
		$button_label = "Submit"; 
		$header = $ml->h1("Quiz");
	}
	else { 
		$button_label = "Vote";  
		$header = $ml->h1("Poll");
	}
	
	$out .= $ml->form(
		$ml->span($poll->{question}, { class => "poll_question" }) . $ml->br() .
		$ml->div($answer_input, { class => "poll_answer_group" }) .
		$ml->input(undef,{ type => "submit", value => $msg{$button_label} }) . $results_link,
		{ action => $this->link(pid => $poll_id, cmd => "display_results"), method => "post" }
	);
	
	return $header . $out;
}

sub display_results {
	my $this = shift;
	my $poll_id = shift;
	my $options = shift;
	my $out;
	
	my $t = new ExSite::Time;
	
	my $poll = $this->{DB}->fetch("poll", $poll_id);
	
	my $member_answer = @{$this->{DB}->fetch_match("poll_response", 
		{ poll_id => $poll_id, member_id => $this->{DB}->my_uid() })}[0];
		
	if ($this->can_vote($poll, $member_answer) && $this->{input}{poll_answer}) {
		if ($member_answer->{poll_response_id}) {
			$member_answer->{answer} = $this->{input}{poll_answer};
			$this->{DB}->update("poll_response", $member_answer);
		}
		else {
			my $uid = $this->{DB}->my_uid();
			my $uid_sig;

			if ($uid) {
				$uid_sig = "-member_id=" . $uid;
			}

			my $poll_response = {
				poll_id => $poll_id,
				answer => $this->{input}{poll_answer},
				signature => $ENV{REMOTE_ADDR} . $ENV{HTTP_USER_AGENT} . $uid_sig
			};
			
			$poll_response->{member_id} = $uid if ($uid);
			
			$this->{DB}->insert("poll_response", $poll_response);		
		}
	}
	elsif ($this->{input}{poll_answer}) {
		$out .= $ml->p($msg{"Sorry, you cannot vote on this poll right now."}, { class => "poll_error" });
	}
	
	if ($poll->{solution}) {
		if ($share{Page} && $this->{input}{poll_answer}) {
			if ($poll->{solution} ne $this->{input}{poll_answer}) { $out .= $ml->div($msg{"You are incorrect."}, { class => "poll_incorrect_answer" }); }
			else { $out .= $ml->div($msg{"You are correct."}, { class => "poll_correct_answer" }); }
		}
		
		$out .= $ml->br();
		$out .= $ml->div($msg{"The correct answer is: " . $poll->{solution}}, { class => "poll_correct_answer" });
	}
	$out .= $ml->p($poll->{message}, { class => "poll_thankyou" });
	
	if ($poll->{secret} eq "N" || $options->{secret} eq "N") {
		my $results;
		my $answers;
		my $total;
		my $responses;
		
		my @good_answers = split(/\|/, $poll->{answers});
		
		foreach my $good_answer (@good_answers) {
			$answers->{$good_answer}{count} = 0;
		}
		
		if ($poll->{result_type} eq "strict" || $options->{result_type} eq "strict") {
			$responses = $this->{DB}->custom_query("SELECT * FROM poll_response WHERE poll_id = $poll_id GROUP BY signature");
		}
		else {
			$responses = $this->{DB}->fetch_match("poll_response", { poll_id => $poll_id });
		}
		
		foreach my $response (@$responses) {
			$answers->{$response->{answer}}{count}++;
			$total++;
		}
				
		my $record_type;
		if ($poll->{solution}) { $record_type = "submissions"; }
		else { $record_type = "votes"; }
		return "No $record_type found." if (!$total);
		
		foreach my $answer (@good_answers) {
			my $vote_percent = int(($answers->{$answer}{count} / $total) * 100);
			my $width = $vote_percent * $config{Poll}{bar}{pixel_per_percent};
			$width .= "px";
			my $max_width = 100 * $config{Poll}{bar}{pixel_per_percent};
			$max_width .= "px";
			my $height = $config{Poll}{bar}{height};
			$height .= "px";
			my $poll_color = $config{Poll}{bar}{color};
			my $poll_votes = $ml->td($answers->{$answer}{count} . " votes", { class => "poll_votes" })
				if (!$config{Poll}{hide_votes_number} || $options->{show_votes_number});
			my $poll_percent = $ml->td($vote_percent . "%", { class => "poll_percentage" })
				if (!$config{Poll}{hide_votes_percent} || $options->{show_votes_percent});
			
			$results .= $ml->tr(
				$ml->td($answer, { class => "poll_answer" }) .
				$poll_votes .
				$poll_percent .
				$ml->td($ml->div(undef, { style => "background-color: $poll_color; width: $width; height: $height;" }), { style => "width: $max_width;" }),
				{ class => "poll_answer_row" }
			);
		}
		
		$out .= $ml->p(
			"Poll Results as of: " . $t->write("date_long") .
			$ml->p($poll->{question}, { class => "poll_question" }) .
			$ml->table($results, { class => "poll_answer_table" }), 
			{ class => "poll_results" }
		);
	}
	
	my $header;
	if ($poll->{solution}) { $header = $ml->h1("Quiz"); }
	else { $header = $ml->h1("Poll"); }	
	
	return $header . $out;
}

sub can_vote {
	my $this = shift;
	my $poll = shift;
	my $member_answer = shift;
	
	my @answers = split(/\|/, $poll->{answers});
	my $good_answer;
	foreach my $answer (@answers) {
		if ($answer eq $this->{input}{poll_answer}) {
			$good_answer = 1;
			last;
		}
	}
	
	return 0 if (!$good_answer);
	return 0 if (!$this->is_poll_active($poll));
	return 0 if (!$this->{DB}->my_uid() && $poll->{privacy} eq "members only");
	return 0 if ($member_answer->{poll_response_id} && !$this->{config}{member_vote_update});
	
	return 1;
}

sub is_poll_active {
	my $this = shift;
	my $poll = shift;
		
	my $t = new ExSite::Time;
	
	if (($poll->{start_date} eq "0000-00-00" && $poll->{end_date} eq "0000-00-00") || 
		(($t->compare($poll->{start_date}, "sql_date") <= 0 && $t->compare($poll->{end_date}, "sql_date") >= 0))) {
		return 1;
	}
	
	return 0;
}

sub vote_report {
	my $this = shift;
	
	my $poll_id = $this->{input}{pid};
	my $poll = $this->{DB}->fetch("poll", $poll_id);
	my @responses = $this->{DB}->fetch_match("poll_response", { poll_id => $poll_id }, "timestamp desc");
	my @strict_responses = $this->{DB}->custom_query("SELECT * FROM poll_response WHERE poll_id = $poll_id GROUP BY signature ORDER BY timestamp desc");

	my $member_label;
	my $submission_type_label;
	if ($poll->{solution}) {
		$member_label = "Submitter";
		$submission_type_label = "submissions";
	}
	else {
		$member_label = "Voter";
		$submission_type_label = "votes";
	}
	
	my @headers = ($member_label, "Answer", "Date Submitted");
	
	if ( $this->{input}{excel} ) {
		if ($this->{input}{report_type} eq "all") { 
			return $this->export_report("Report All - " . $poll->{question}, \@responses); 
		}
		else { 
			return $this->export_report("Report Strict - " . $poll->{question}, \@strict_responses); 
		}
	}	
	
	my $report_all = new ExSite::ReportBuilder();		
	$report_all->init(title => "Report All",
		headers => \@headers,
		nodata => "No $submission_type_label found.",
		tools => $this->export_excel("all", \@responses),
	);
	foreach my $response (@responses) 
	{
		my @row = $this->report_voter_row($response);
		$report_all->push(@row) if (scalar @row);
	}
	
	my $report_strict = new ExSite::ReportBuilder();
	$report_strict->init(title => "Report Strict",
		headers => \@headers,
		nodata => "No $submission_type_label found.",
		tools => $this->export_excel("strict", \@strict_responses),
	);
	foreach my $strict_response (@strict_responses) 
	{
		my @row = $this->report_voter_row($strict_response);
		$report_strict->push(@row) if (scalar @row);
	}
	
	my $duplicate_count = scalar @responses - scalar @strict_responses;

	my $report_all_pane = $ml->table(
		$ml->tr(
			$ml->td($report_all->make, { style => "vertical-align: top; padding-right: 20px;" }) .
			$ml->td($this->display_results($poll->{poll_id}, { result_type => "all", secret => "N", show_votes_number => 1, show_votes_percent => 1 }), 
				{ style => "vertical-align: top;" })
		)
	);	
	my $report_strict_pane = $ml->table(
		$ml->tr(
			$ml->td($report_strict->make, { style => "vertical-align: top; padding-right: 20px;" }) .
			$ml->td($this->display_results($poll->{poll_id}, { result_type => "strict", secret => "N", show_votes_number => 1, show_votes_percent => 1 }) .
			$ml->br() . $ml->b($duplicate_count . " duplicate $submission_type_label removed"), 
				{ style => "vertical-align: top;" })
		)
	);

	my $out = &ExSite::HTML::DynTabBox(
			tabs=>["Report All","Report Strict"],
			panes=>[$report_all_pane,
				$report_strict_pane],
			width=>"100%",
			thistab=>"Report All"
	);
	
	return $ml->h1($poll->{question}) . $out;
}

sub export_report {
	my $this = shift;
	my $report_header = shift;
	my $responses = shift;
	my $headers = shift;
	
	my $data;
	foreach my $response (@$responses) {
		my $member = $this->{DB}->fetch("member", $response->{member_id});
		my $member_name;
		my $member_id = $response->{member_id} . ": ";
		
		if ($member->{name}) {
			$member_name = $member->{name};
		}
		elsif ($member->{first_name} || $member->{last_name}) {
			$member_name = $member->{first_name} . " " . $member->{last_name};
		}
		else {
			$member_name = "anonymous";
			$member_id = "";
		}
	
		$response->{member_id} = $member_id.$member_name;
		
		my $t = new ExSite::Time($response->{timestamp}, "sql_timestamp");
		
		$response->{timestamp} = $t->write("datetime2");
		
		push(@$data, $response);
	}
	
	my $cols;
        my $ncol = $this->{DB}->{map}->get_ncol("poll_response");
	for (my $icol = 0; $icol < $ncol; $icol++) {
		my %colmap = $this->{DB}->{map}->get_column("poll_response",$icol);
		my $read = $colmap{read};
		
		if ($this->{DB}->authorize() >= $read) {
			my $label = $colmap{label};
			push(@$cols, "$label;" . $colmap{column});
		}
	}
        
        $report_header =~ s/\s/_/g;
        $report_header =~ s/\?//g;
        
        require ExSite::Excel;
        print "content-type: application/vnd.ms-excel\ncontent-disposition: attachment; filename=$report_header.xls\n\n";
        &ExSite::Excel::ExcelExport($data, "-", $cols);
        
        exit 0;
}

sub report_voter_row {
	my ($this,$voter_row) = @_;
	
	my $t = new ExSite::Time($voter_row->{timestamp}, "sql_timestamp");
	my $member = $this->{DB}->fetch("member", $voter_row->{member_id});
	
	my $member_name;
	my $member_id = $voter_row->{member_id} . ": ";
	if ($member->{name}) {
		$member_name = $member->{name};
	}
	elsif ($member->{first_name} || $member->{last_name}) {
		$member_name = $member->{first_name} . " " . $member->{last_name};
	}
	else {
		$member_name = "anonymous";
		$member_id = "";
	}
	
	return (
		$member_id.$member_name,
		$voter_row->{answer},
		$t->write("datetime2")
	);
}

sub export_excel {
	my $this = shift;
	my $report_type = shift;
	my $objects = shift;
	my $num = scalar(@$objects);
	my $in = $this->{input};

	my %params;
	
	foreach my $param (keys %$in) {
		$params{$param} = $in->{$param};
	}
	$params{excel} = 1;
	$params{report_type} = $report_type;

	my $export_link = $this->{ML}->a(
		"export to Excel", {
			href => $this->link(%params),
			target => "_blank"
		} 
	);

	return "No records found." if (! $num );
	return "$num records found - $export_link";
}

1;
