package Modules::Preferences;

#----------------------------------------------------------------------------
#
#   Copyright (C) 2009 - Exware Solutions, Inc.  http://www.exware.com
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------
#
#  Preference Manager
#
#----------------------------------------------------------------------------

use strict;

# useful kernel libs; you can/should remove any of these that are not needed

use ExSite::Config;          # required
use ExSite::Input;           # optional input manager
use ExSite::Misc;            # optional utils
use ExSite::Util;            # optional utils
use ExSite::HTML;
use ExSite::ML;              # optional markup-language generation
use ExSite::Attribute;

# recommended base class for plug-in modules

use Modules::BaseDCD;

# declare package globals

use vars qw(@ISA $ml);

# define our class inheritance

@ISA = qw(Modules::BaseDCD); # inherit from this base class

# write method (builds output for regular web pages)

sub write {
    my ($this,$options) = @_;
    return;
}

# ioctl method (used by ExSite to query the module for its functionality)

sub ioctl {
    my $this = shift;
    $_ = shift;           # $_ is the ioctl request

    if (/isRestricted/) {
	return 0;
    }
    elsif (/isService/) {
	return 0;
    }
    elsif (/ModuleName/) {
	return "Preferences";
    }
    elsif (/ModuleInfo/) {
	return "Preference Manager";
    }
    elsif (/Category/) {
	return "System";
    }
    elsif (/ControlPanel/) {
	return \&ctrl_panel;
    }
    elsif (/Summary/) {
	return \&summary;
    }
}

#----------------------------------------------------------------------------
# Everything after this point consists of private methods.

# ctrl_panel() generates the contents of the administrator control panel

sub ctrl_panel {
    my $this = shift;

    # declare a markup generator
    $ml = new ExSite::ML;

    my $sid = $this->get_section_id();
    my $out;
    if ($sid) {
	my $s = new ExSite::Section(id=>$sid);
	$out .= $ml->h1($s->title()." Preferences");
	my $in = new ExSite::Input;
	my $post = $in->post();
	if (keys %$post > 0) {
	    $out .= $this->update_preferences(%$post);
	}
	my $f = $share{DB}->form(method=>"post");
	my $pref = new ExSite::Attribute("section",$sid);
	$pref->load();
	my @metafields = $pref->get_allowed();
	if (@metafields == 0) {
	    my %meta = $pref->get_all();
	    @metafields = keys %meta;
	}
	my (@title,@pane,@state);
	if (@metafields > 0) {
	    foreach my $key (sort @metafields) {
		$share{DB}->form(method=>"post");
		$pref->input_record({name=>$key});
		$share{DB}->form()->set("buttons",$ml->input(undef,{type=>"submit",value=>"Update"}));
		push @title, $key;
		push @pane, $share{DB}->form()->make;
		push @state, 0;
#		$out .= &ExSite::HTML::BasicBox(title=>$key,pane=>$share{DB}->form()->make);
	    }
	    $out .= &ExSite::HTML::AccordionBox(titles=>\@title,panes=>\@pane,states=>\@state);
	    $out .= $ml->p("Click on a setting to update it. Please update only one preference setting at a time.");
	}
	else {
	    $out .= $this->warn("No preferences settings available.");
	}
    }
    else {
	$out = $this->set_section_id();
    }

    return $out;
}

sub ml {
	my ($this) = @_;
	return &get_obj("ML");
}

sub update_preferences {
    my ($this,%data) = @_;
    my $db = $share{DB};
    my $errcnt = &AtError();
    $db->get_data();
    $db->do_db($db->get_action());
    my $out;
    # report results
    if ($errcnt) {
	# uh-oh!
	$out = $this->ml->h3("Problem:");
	$out .= $this->show_diagnostics("error","html");
	$out .= $this->ml->p($this->ml->a("Go back to form.",{href=>"javascript:history.back()"}));
    }
    else {
	# success; generic confirmation message
	$out = $ml->h2("Preferences were updated!",{class=>"help"});
    }
    return $out;
}

sub summary {
    my $this = shift;
    my $r = new ExSite::ReportBuilder(title=>"Preferences");
    $r->headers(["Setting","Value"]);
    my $sid = $this->get_section_id();
    if ($sid) {
	my $s = new ExSite::Section(id=>$sid);
	my $pref = new ExSite::Attribute("section",$sid);
	$pref->load();
	my @metafields = $pref->get_allowed();
	if (@metafields == 0) {
	    my %meta = $pref->get_all();
	    @metafields = keys %meta;
	}
	if (@metafields > 0) {
	    foreach my $key (sort @metafields) {
		$r->push($pref->label($key),$pref->showdata($key));
	    }
	}
    }
    return $r->make();
}

1;
