package Modules::Subscribe;

#----------------------------------------------------------------------------
#
#   Copyright (C) 2010 - Matthew Wong
#
#   This file is part of ExSite WebWare (ExSite, for short).
#
#   ExSite is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   ExSite is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with ExSite; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#   Users requiring warranty coverage and/or support may arrange alternate
#   commercial licensing for ExSite, by contacting Exware Solutions 
#   via the website noted above.
#
#----------------------------------------------------------------------------

use strict;

# useful kernel libs; you can/should remove any of these that are not needed

use ExSite::Config;          # required
use ExSite::Captcha;
use ExSite::Input;           # optional input manager
use ExSite::Misc;            # optional utils
use ExSite::Util;            # optional utils
use ExSite::ML;              # optional markup-language generation
use ExSite::FormBuilder;     # optional form generation
use ExSite::ReportBuilder;   # optional report generation
use ExSite::Form;            # optional database manager

# recommended base class for plug-in modules

use Modules::BaseDCD;

# declare package globals

use vars qw(@ISA $ml);       # Perl inheritance global & markup language object

# define our class inheritance

@ISA = qw(Modules::BaseDCD); # inherit from this base class

# ioctl method (used by ExSite to query the module for its functionality)

sub ioctl {
	my $this = shift;
	$_ = shift;           # $_ is the ioctl request

	if (/isService/) {
		return 1;
	}
	elsif (/ModuleName/) {
		return "Subscribe";
	}
	elsif (/isRestricted/) {
		return $config{Subscriber}{isRestricted};
	}
	elsif (/DynContentParameter/) {
		return "<p><select name=\"param\">
			<option value=\"\">Subscribe Form</option>
			<option value=\"cmd=unsubscribe_form\">Unsubscribe Form</option>
			</select></p>\n";
	}
}

# write method (builds output for regular web pages)

sub write {
	my ($this,$options) = @_;

	$ml = new ExSite::ML;
	my %opt = &DecodeString($options);
	my $cmd = $opt{cmd} || $this->{input}{cmd};
	my $out;

	if ($cmd =~ /^(unsubscribe_form)$/) {
		$out .= $this->$cmd(\%opt);
	}
	else {
		$out .= $this->subscriber_form(\%opt);
	}

	return $this->wrap($out);
}

sub add_subscriber {
	my $this = shift;
	my $opt = shift;
	my $out; 
	
	my $contact_record = {
		section_id => $share{DB}->this_site->{section_id},
		type => $opt->{contact_type} || $config{Subscribe}{contact_type} || "subscription",
		info => $this->{input}{name},
		address => $this->{input}{address},
		city => $this->{input}{city},
		provstate => $this->{input}{provstate},
		country => $this->{input}{country},
		pcode => $this->{input}{pcode},
		phone1 => $this->{input}{phone1},
		phone2 => $this->{input}{phone2},
		fax => $this->{input}{fax},
		email => $this->{input}{email},
		web => $this->{input}{website},
		privacy => $config{Subscribe}{privacy} || "administrators",
		subscribe => "Y",
		category => $opt->{contact_category} || $config{Subscribe}{contact_category} || "subscription"
	};
		
	# validate the email, captcha, and for dupes
	my $captcha = new ExSite::Captcha(mode => "passive");	
	if (!$captcha->pass()){
		return $ml->p($msg{"Error detected, please go back and try again."} . $ml->br() . 
			$ml->input(undef,{type=>"button",value=>"&lt;&lt; " . $msg{"Back"},onClick=>"history.back()"}));
	}		
	
	if ($contact_record->{email} !~ /@.+\.\w+$/) {
		return $ml->p($msg{"Email address formatted incorrectly, please go back and try again."} . $ml->br() . 
			$ml->input(undef,{type=>"button",value=>"&lt;&lt; " . $msg{"Back"},onClick=>"history.back()"}));
	}
	
	my $existing_contact_record = @{$share{DB}->fetch_match("contact", 
		{ email => $this->{input}{email}, category => $contact_record->{category}, 
		type => $contact_record->{type}, section_id => $contact_record->{section_id} })}[0];

	if ($existing_contact_record) {
		$share{DB}->update("contact", $contact_record, { contact_id => $existing_contact_record->{contact_id} });
	}
	else {
		$share{DB}->insert("contact", $contact_record);
	}
	
	$out .= $config{Subscribe}{thank_you_message} || "Thank you for subscribing!";
	
	return $msg{$out};
}

sub unsubscribe_form {
	my $this = shift;
	my $opt = shift;
	my $out;

	if ($this->{input}{doform}) {
		return $this->unsubscribe_subscriber($opt);
	}

	my $f = new ExSite::FormBuilder(action=>$this->link(doform=>1,cmd=>"unsubscribe_form"),method => "POST");

	$f->input(
		type => "text",
		name => "email",
		prompt => $msg{"Email"},
		required => 1,
		size => 40
	);
	
	my $captcha = new ExSite::Captcha(mode => "passive");
	$f->input(
		type => "preformatted",
		name => "captcha",
		input => $captcha->make(),
	);	
	
	$out .= $f->make;

	return $out;
}

sub unsubscribe_subscriber {
	my $this = shift;
	my $opt = shift;
	my $out;
	
	# validate the email and captcha
	my $captcha = new ExSite::Captcha(mode => "passive");	
	if (!$captcha->pass()){
		return $ml->p($msg{"Error detected, please go back and try again."} . $ml->br() . 
			$ml->input(undef,{type=>"button",value=>"&lt;&lt; " . $msg{"Back"},onClick=>"history.back()"}));
	}	
	
	if ($this->{input}{email} !~ /@.+\.\w+$/) {
		return $ml->p($msg{"Email address formatted incorrectly, please go back and try again."} . $ml->br() . 
			$ml->input(undef,{type=>"button",value=>"&lt;&lt; " . $msg{"Back"},onClick=>"history.back()"}));
	}	
	
	my $section_id = $share{DB}->this_site->{section_id};
	my $type = $opt->{contact_type} || $config{Subscribe}{contact_type} || "subscription";
	my $category = $opt->{contact_category} || $config{Subscribe}{contact_category} || "subscription";
	
	my $existing_contact_record = @{$share{DB}->fetch_match("contact", 
		{ email => $this->{input}{email}, category => $category, type => $type, section_id => $section_id })}[0];
		
	if ($existing_contact_record) {
		$share{DB}->update("contact", { subscribe => "N" }, { contact_id => $existing_contact_record->{contact_id} });		
	}
	
	$out .= $config{Subscribe}{unsubscribe_message} || $ml->h1("Thank You") . "You have been unsubscribed.";
		
	return $msg{$out};
}

# last item: clean default template
sub subscriber_form {
	my $this = shift;
	my $opt = shift;
	my $out;

	if ($this->{input}{doform}) {
		return $this->add_subscriber($opt);
	}

	my $f = new ExSite::FormBuilder(action=>$this->link(doform=>1,cmd=>"subscriber_form"),method=>"POST");

	my $form_template = $config{Subscribe}{subscriber_form}{template} || <<END;
<p class="name">[[name:prompt]]<br>[[name:input]]</p>
<p class="email">[[email:prompt]]<br>[[email:input]]</p>
END

	$f->template($form_template);

	$f->input(
		type => "text",
		name => "name",
		prompt => $msg{"Name"},
		required => 1,
		size => 40
	);

	$f->input(
		type => "text",
		name => "email",
		prompt => $msg{"Email"},
		required => 1,
		size => 40
	);

	if ($config{Subscribe}{subscriber_form}{show_address}) {
		$f->input(
			type => "textarea",
			name => "address",
			prompt => $msg{"Address"},
			size => 80
		);
	}

	if ($config{Subscribe}{subscriber_form}{show_city}) {
		$f->input(
			type => "text",
			name => "city",
			prompt => $msg{"City"},
			size => 40
		);
	}

	if ($config{Subscribe}{subscriber_form}{show_provstate}) {
		$f->input(
			input => $share{DB}->input_exsite(
				name => "provstate",
				datatype => "list:provstate"),
			name => "provstate",
			prompt => $msg{$config{Subscribe}{provstate_label}} || $msg{"Province/State"}
		);
	}

	if ($config{Subscribe}{subscriber_form}{show_country}) {
		$f->input(
			input => $share{DB}->input_exsite(
				name => "country",
				datatype => "list:country"),
			name => "country",
			prompt => $msg{"Country"}
		);
	}
	
	if ($config{Subscribe}{subscriber_form}{show_postal_code}) {
		$f->input(
			type => "text",
			name => "pcode",
			prompt => $msg{$config{Subscribe}{postal_code_label}} || $msg{"Postal Code"},
			size => 10
		);
	}	
	
	if ($config{Subscribe}{subscriber_form}{show_phone1}) {
		$f->input(
			type => "text",
			name => "phone1",
			prompt => $msg{$config{Subscribe}{phone1_label}} || $msg{"Phone"},
			size => 20
		);
	}
	
	if ($config{Subscribe}{subscriber_form}{show_phone2}) {
		$f->input(
			type => "text",
			name => "phone2",
			prompt => $msg{$config{Subscribe}{phone2_label}} || $msg{"Alt. Phone"},
			size => 20
		);
	}	
	
	if ($config{Subscribe}{subscriber_form}{show_fax}) {
		$f->input(
			type => "text",
			name => "fax",
			prompt => $msg{"Fax"},
			size => 20
		);
	}

	if ($config{Subscribe}{subscriber_form}{show_website}) {
		$f->input(
			type => "text",
			name => "website",
			prompt => $msg{"Website"},
			size => 40
		);
	}

	my $captcha = new ExSite::Captcha(mode => "passive");
	$f->input(
		type => "preformatted",
		name => "captcha",
		input => $captcha->make(),
	);

	$out .= $f->make;	

	return $out;
}

sub wrap {
	my ($this,$body) = @_;
	
	my $out .= $ml->link(undef, {
		rel => "stylesheet",
		type => "text/css",
		href => "$config{server}{server}$config{server}{HTMLpath}/_Modules/Subscribe/Subscribe.css",
	});
	$out.= $body;
	
	return $ml->div($out, { class => "Subscribe" } );
}


1;

